/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
 * (c) Copyright 2000-2002 The MITRE Corporation (MITRE)
 *
 * Permission is hereby granted, without payment, to copy, use,
 * modify, display and distribute this software and its documentation,
 * if any, for any purpose, provided, first, that the US Government
 * and any of its agencies will not be charged any license fee and/or
 * royalties for the use of or access to said copyright software, and
 * provided further that the above copyright notice and the following
 * three paragraphs shall appear in all copies of this software,
 * including derivatives utilizing any portion of the copyright
 * software.  Use of this software constitutes acceptance of these
 * terms and conditions.
 *
 * IN NO EVENT SHALL MITRE BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT,SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF MITRE
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * MITRE SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTIES
 * INCLUDING,BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT.
 *
 * THE SOFTWARE IS PROVIDED "AS IS."  MITRE HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package org.mitre.audiogui;

import java.io.IOException;
import java.net.Socket;
import galaxy.lang.GFrame;
import galaxy.lang.Clause;
import galaxy.server.Server;

public class AudioGUIServer extends Server
{
    private Controller controller = null;
    
    public AudioGUIServer(AudioGUIMainServer mainServer, Socket socket) 
	throws IOException 
    {
	super(mainServer,socket);
	controller = mainServer.getController();
	controller.setServer(this);
    }
    
    //-------------------------------------------------------------------------
    // Dispatch methods.
    //-------------------------------------------------------------------------

    public void serverOpReinitialize(GFrame frame) 
    {
	GFrame initFrame = new Clause("initialize_audio_gui");
	try {
	    getCurrentEnvironment().writeFrame(initFrame);
	} catch (IOException ioex) {
	    logErrorMessage("Error writing frame.", ioex, "AudioGUIServer.serverOpReinitialize(GFrame)");
	}    
    }
    
    public GFrame serverOpUpdate(GFrame frame)
    { 
	Boolean status = null;
	Boolean queryStatus = null;

	status = frame.getIntAsBoolean(":server_response");
	queryStatus = frame.getIntAsBoolean(":query_status");
	if(status != null) {
	    if(status.booleanValue()) {
		if(queryStatus != null && queryStatus.booleanValue()) {
		    // Handle response to initialize_audio_gui
		    controller.updateState(frame);
		} else {
		    // Handle response to GUI-initiated audio server change 
		    controller.setReplyFrame(frame);
		    controller.doNotifyAll();
		}
	    } else {
		// Handle unrequested state updates sent from audio server
		controller.updateState(frame);
	    }
	}

	return null;
    } 
}
