/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
 * (c) Copyright 2000-2002 The MITRE Corporation (MITRE)
 *
 * Permission is hereby granted, without payment, to copy, use,
 * modify, display and distribute this software and its documentation,
 * if any, for any purpose, provided, first, that the US Government
 * and any of its agencies will not be charged any license fee and/or
 * royalties for the use of or access to said copyright software, and
 * provided further that the above copyright notice and the following
 * three paragraphs shall appear in all copies of this software,
 * including derivatives utilizing any portion of the copyright
 * software.  Use of this software constitutes acceptance of these
 * terms and conditions.
 *
 * IN NO EVENT SHALL MITRE BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT,SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF MITRE
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * MITRE SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTIES
 * INCLUDING,BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT.
 *
 * THE SOFTWARE IS PROVIDED "AS IS."  MITRE HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package org.mitre.audiogui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.mitre.audiogui.event.*;
import galaxy.util.Logger;

/**
 * This is the main GUI for the audio server controller.
 */
public class ControllerGUI extends JFrame implements ControllerListener 
{
    private Controller controller;
    private boolean debugEnabled;

    // GUI components.
    private JMenuBar jMenuBar1 = new JMenuBar();
    private JMenu jMenu1 = new JMenu();
    private JMenuItem exitMenuItem = new JMenuItem();
    private GridLayout gridLayout = new GridLayout();
    private JPanel panel = new JPanel();
    private JButton trumpButton = new JButton();
    private JLabel vadLabel = new JLabel();
    private JButton resendButton = new JButton();
    private JMenuItem keypadMenuItem = new JMenuItem();
    private JMenuItem debugMenuItem = new JMenuItem();
    private JMenuItem playboxMenuItem = new JMenuItem();
    private JLabel streamingLabel = new JLabel();
    private JPopupMenu vadPopupMenu = new JPopupMenu();
    private JMenuItem vadMenuItem = new JMenuItem();
    private JPopupMenu streamingPopupMenu = new JPopupMenu();
    private JMenuItem streamingMenuItem = new JMenuItem();
    private JPopupMenu bargeInPopupMenu = new JPopupMenu();
    private JMenuItem bargeInMenuItem = new JMenuItem();
    private JLabel bargeInLabel = new JLabel();
    private JPopupMenu duplexPopupMenu = new JPopupMenu();
    private JMenuItem duplexMenuItem = new JMenuItem();
    private JLabel duplexLabel = new JLabel();
    private Logger logger = null;

    /**
     * Creates a controller GUI.
     *
     * @param controller the controller object that manages this GUI
     */
    public ControllerGUI(Controller controller) 
    {
	this.controller = controller;
	logger = Logger.getLogger();
	debugEnabled = controller.isDebugEnabled();
	
	try {
	    jbInit();
	} catch(Exception ex) {
	    logger.logErrorMessage("Error while initializing controller GUI.",
				   ex, "ControllerGUI(Controller)");
	}
	
	controller.addEventListener(this);
	
	update();
    }
    
    //-------------------------------------------------------------------------
    // GUI-related methods.
    //-------------------------------------------------------------------------

    private void jbInit() throws Exception 
    {
	this.setDefaultCloseOperation(3);
	this.setJMenuBar(jMenuBar1);
	this.setTitle("Audio Server GUI");
	jMenu1.setText("Controls");
	exitMenuItem.setText("Exit");
	exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    exitMenuItem_actionPerformed(event);
		}
	    });
	gridLayout.setRows(6);
	gridLayout.setColumns(1);
	panel.setLayout(gridLayout);
	trumpButton.setPreferredSize(new Dimension(280, 27));
	trumpButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    trumpButton_actionPerformed(event);
		}
	    });
	trumpButton.setBackground(Color.green);
	trumpButton.setEnabled(false);
	vadLabel.setHorizontalAlignment(SwingConstants.CENTER);
	vadLabel.addMouseListener(new java.awt.event.MouseAdapter() {
		public void mousePressed(MouseEvent event) {
		    vadLabel_mousePressed(event);
		}
	    });
	resendButton.setActionCommand("resendButton");
	resendButton.setEnabled(false);
	resendButton.setText("Resend");
	resendButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    resendButton_actionPerformed(event);
		}
	    });
	keypadMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    keypadMenuItem_actionPerformed(event);
		}
	    });
	debugMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    debugMenuItem_actionPerformed(event);
		}
	    });
	playboxMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    playboxMenuItem_actionPerformed(event);
		}
	    });
	streamingLabel.setHorizontalAlignment(SwingConstants.CENTER);
	streamingLabel.addMouseListener(new java.awt.event.MouseAdapter() {
		public void mousePressed(MouseEvent event) {
		    streamingLabel_mousePressed(event);
		}
	    });
	vadMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    vadMenuItem_actionPerformed(event);
		}
	    });
	streamingMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    streamingMenuItem_actionPerformed(event);
		}
	    });
	bargeInMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    bargeInMenuItem_actionPerformed(event);
		}
	    });
	bargeInLabel.setHorizontalAlignment(SwingConstants.CENTER);
	bargeInLabel.addMouseListener(new java.awt.event.MouseAdapter() {
		public void mousePressed(MouseEvent event) {
		    bargeInLabel_mousePressed(event);
		}
	    });
	duplexMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    duplexMenuItem_actionPerformed(event);
		}
	    });
	duplexLabel.setHorizontalAlignment(SwingConstants.CENTER);
	duplexLabel.addMouseListener(new java.awt.event.MouseAdapter() {
		public void mousePressed(MouseEvent event) {
		    duplexLabel_mousePressed(event);
		}
	    });
	this.getContentPane().add(panel, BorderLayout.CENTER);
	panel.add(bargeInLabel, null);
	panel.add(duplexLabel, null);
	panel.add(streamingLabel, null);
	panel.add(vadLabel, null);
	panel.add(trumpButton, null);
	panel.add(resendButton, null);
	jMenuBar1.add(jMenu1);
	jMenu1.add(keypadMenuItem);
	jMenu1.add(debugMenuItem);
	jMenu1.add(playboxMenuItem);
	jMenu1.add(exitMenuItem);
	vadPopupMenu.add(vadMenuItem);
	streamingPopupMenu.add(streamingMenuItem);
	bargeInPopupMenu.add(bargeInMenuItem);
	duplexPopupMenu.add(duplexMenuItem);
    }

    private void displayErrorMessage(String message)
    {
	JOptionPane.showMessageDialog(this, message, "Controller Error",
				      JOptionPane.ERROR_MESSAGE);
    }

    private void update() 
    {
	updateBargeInLabel();
	updateBargeInMenuItem();
	updateDebugMenuItem();
	updateDuplexLabel();
	updateDuplexMenuItem();
	updateKeypadMenuItem();
	updatePlayboxMenuItem();
	updateStreamingMenuItem();
	updateVADLabel();
	updateVADMenuItem();
	updateTrumpButton();
	updateResendButton();
	updateStreamingLabel();
    }

    private void updateBargeInLabel() 
    {
	bargeInLabel.setText(controller.isBargeInEnabled() ? "Barge-in Enabled" : "Barge-in Disabled");
    }
    
    private void updateDuplexLabel() 
    {
	duplexLabel.setText(controller.isHalfDuplexEnabled() ? "Half-duplex Mode" : "Full-duplex Mode");
    }
    
    private void updateBargeInMenuItem() 
    {
	bargeInMenuItem.setText(controller.isBargeInEnabled() ? "Disable Barge-in" : "Enable Barge-in");
    }
    
    private void updateDebugMenuItem() 
    {
	debugMenuItem.setText(debugEnabled ? "Disable Debugging Output" : "Enable Debugging Output");
    }
    
    private void updateDuplexMenuItem() 
    {
	duplexMenuItem.setText(controller.isHalfDuplexEnabled() ? "Switch to Full-Duplex" : "Switch to Half-Duplex");
    }
    
    private void updateKeypadMenuItem() 
    {
	keypadMenuItem.setText(controller.isKeypadVisible() ? "Hide Keypad" : "Show Keypad");
	keypadMenuItem.setEnabled(controller.isKeypadEnabled());
    }
    
    private void updatePlayboxMenuItem() 
    {
	playboxMenuItem.setText(controller.isPlayboxEnabled() ? "Disable Playbox Brokering" : "Enable Playbox Brokering");
    }
    
    private void updateStreamingMenuItem() 
    {
	streamingMenuItem.setText(controller.isStreamingEnabled() ?
				  "Turn Off Streaming" : "Turn On Streaming");
	streamingMenuItem.setEnabled(!controller.isRecordStarted());
    }
    
    private void updateVADMenuItem() 
    {
	vadMenuItem.setText(controller.isPTTEnabled() ? "Switch to VAD Mode" : "Switch to PTT Mode");
    }
    
    private void updateResendButton() 
    {
	resendButton.setEnabled(!controller.isRecordStarted() && controller.isAudioResendable());
    }
    
    private void updateStreamingLabel() 
    {
	streamingLabel.setText(controller.isStreamingEnabled() ? "Streaming Mode" : "Batch Mode");
    }
    
    private void updateVADLabel() 
    {
	if(controller.isPTTEnabled())
	    vadLabel.setText(controller.isRecordStarted() ? "PTT Mode: Recording" : "PTT Mode: Ready to record");
	else
	    vadLabel.setText(controller.isMuteActive() ? "VAD Mode: Muted" : "VAD Mode: Active");
    }
    
    private void updateTrumpButton() 
    {
	if(controller.isPTTEnabled()) {
	    trumpButton.setText(controller.isRecordStarted() ? "Click to Stop Recording" :
				"Click to Start Recording");
	    trumpButton.setEnabled(controller.isRecordStarted() || controller.isRecordStartable());
	    trumpButton.setBackground(controller.isRecordStarted() ? Color.red : Color.green);
	} else {
	    trumpButton.setText(controller.isMuteActive() ? "Click to Un-Mute" : "Click to Mute");
	    trumpButton.setEnabled(!controller.isOnHook());
	    trumpButton.setBackground(controller.isVoiceActive() ? Color.red : Color.green);
	}
    }
    
    private void resendButton_actionPerformed(ActionEvent event) 
    {
	if(!controller.resend())
	    displayErrorMessage("Could not resend audio data.");
    }
    
    private void trumpButton_actionPerformed(ActionEvent event) 
    {
	if (controller.isPTTEnabled()) {
	    if(!controller.setRecordStarted(!controller.isRecordStarted()))
		displayErrorMessage("Could not change recording status.");
	} else {
	    if(!controller.setMuteActive(!controller.isMuteActive()))
		displayErrorMessage("Could not change mute status.");
	}
    }
    
    private void bargeInMenuItem_actionPerformed(ActionEvent event) 
    {
	if(!controller.setBargeInEnabled(!controller.isBargeInEnabled()))
	    displayErrorMessage("Could not change barge-in setting.");
    }
    
    private void debugMenuItem_actionPerformed(ActionEvent event) 
    {
	controller.setDebugEnabled(!debugEnabled);
    }
    
    private void exitMenuItem_actionPerformed(ActionEvent event) 
    {
	System.exit(0);
    }
    
    private void duplexMenuItem_actionPerformed(ActionEvent event) 
    {
	if(!controller.setHalfDuplexEnabled(!controller.isHalfDuplexEnabled()))
	    displayErrorMessage("Could not change duplex setting.");
    }
    
    private void keypadMenuItem_actionPerformed(ActionEvent event) 
    {
	controller.setKeypadVisible(!controller.isKeypadVisible());
    }
    
    private void streamingMenuItem_actionPerformed(ActionEvent event) 
    {
	if(!controller.setStreaming(!controller.isStreamingEnabled()))
	    displayErrorMessage("Could not change streaming setting.");
    }
    
    private void vadMenuItem_actionPerformed(ActionEvent event) 
    {
	if(!controller.setPTTEnabled(!controller.isPTTEnabled()))
	    displayErrorMessage("Could not change PTT setting.");
    }
    
    private void playboxMenuItem_actionPerformed(ActionEvent event) 
    {
	if(!controller.setPlayboxEnabled(!controller.isPlayboxEnabled()))
	    displayErrorMessage("Could not change playbox setting.");
    }
    
    private void vadLabel_mousePressed(MouseEvent event) 
    {
	vadPopupMenu.show(vadLabel,event.getX()-5,event.getY()-5);
	vadPopupMenu.setVisible(true);
    }
    
    private void streamingLabel_mousePressed(MouseEvent event) 
    {
	streamingPopupMenu.show(streamingLabel,event.getX()-5,event.getY()-5);
	streamingPopupMenu.setVisible(true);
    }
    
    private void bargeInLabel_mousePressed(MouseEvent event) 
    {
	bargeInPopupMenu.show(bargeInLabel,event.getX()-5,event.getY()-5);
	bargeInPopupMenu.setVisible(true);
    }
    
    private void duplexLabel_mousePressed(MouseEvent event) 
    {
	duplexPopupMenu.show(duplexLabel,event.getX()-5,event.getY()-5);
	duplexPopupMenu.setVisible(true);
    }
    
    //-------------------------------------------------------------------------
    // Implementation of ControllerListener interface
    //-------------------------------------------------------------------------

    public void eventGenerated(UpdateEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: ControllerGUI handling " + event.toString());
	
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    update();
		}
	    });
    }
    
    public void eventGenerated(GUIEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: ControllerGUI handling " + event.toString());	
	
	switch(event.getID()) {
	case GUIEvent.KEYPAD_INVISIBLE: case GUIEvent.KEYPAD_VISIBLE:
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			updateKeypadMenuItem();
		    }
		});
	    break;
	case GUIEvent.GUI_INVISIBLE: case GUIEvent.GUI_VISIBLE:
	    setVisible(controller.isGUIVisible());
	    break;
	}
    }
    
    public void eventGenerated(KeypadEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: ControllerGUI handling " + event.toString());

	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    updateTrumpButton();
		}
	    });
    }
    
    public void eventGenerated(ModeEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: ControllerGUI handling " + event.toString());
	
	switch (event.getID()) {
	case ModeEvent.DEBUG_DISABLED:
	    debugEnabled = false;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			updateDebugMenuItem();
		    }
		});
	    break;
	case ModeEvent.DEBUG_ENABLED:
	    debugEnabled = true;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			updateDebugMenuItem();
		    }
		});
	break;
	}
    }
}
