/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
 * (c) Copyright 2000-2002 The MITRE Corporation (MITRE)
 *
 * Permission is hereby granted, without payment, to copy, use,
 * modify, display and distribute this software and its documentation,
 * if any, for any purpose, provided, first, that the US Government
 * and any of its agencies will not be charged any license fee and/or
 * royalties for the use of or access to said copyright software, and
 * provided further that the above copyright notice and the following
 * three paragraphs shall appear in all copies of this software,
 * including derivatives utilizing any portion of the copyright
 * software.  Use of this software constitutes acceptance of these
 * terms and conditions.
 *
 * IN NO EVENT SHALL MITRE BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT,SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF MITRE
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * MITRE SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTIES
 * INCLUDING,BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT.
 *
 * THE SOFTWARE IS PROVIDED "AS IS."  MITRE HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package org.mitre.audiogui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.mitre.audiogui.event.*;
import galaxy.util.Logger;

/**
 * This is a simple GUI that represents a touchtone keypad.
 */
public class KeypadGUI extends JFrame implements ControllerListener, KeyListener
{
    private Controller controller;
    private boolean debugEnabled;
    private Logger logger = null;

    // GUI components.
    private JPanel jPanel1 = new JPanel();
    private GridLayout gridLayout1 = new GridLayout();
    private JButton key1 = new JButton();
    private JButton keyHash = new JButton();
    private JButton key0 = new JButton();
    private JButton keyStar = new JButton();
    private JButton key9 = new JButton();
    private JButton key8 = new JButton();
    private JButton key7 = new JButton();
    private JButton key6 = new JButton();
    private JButton key5 = new JButton();
    private JButton key4 = new JButton();
    private JButton key3 = new JButton();
    private JButton key2 = new JButton();
    private JPanel jPanel2 = new JPanel();
    private JButton hookButton = new JButton();
    private BorderLayout borderLayout1 = new BorderLayout();
    private JButton flashButton = new JButton();
    private JMenuBar fileMenuBar = new JMenuBar();
    private JMenu fileMenu = new JMenu();
    private JMenuItem debugMenuItem = new JMenuItem();
    private JMenuItem exitMenuItem = new JMenuItem();
    
    /**
     * Creates a keypad GUI.
     *
     * @param controller the controller object that manages this GUI
     * @param setMenuBar if true, this GUI's menubar will be enabled
     */
    public KeypadGUI(Controller controller, boolean setMenuBar) 
    {
	this.controller = controller;
	logger = Logger.getLogger();
	debugEnabled = controller.isDebugEnabled();
	
	try {
	    jbInit(setMenuBar);
	} catch(Exception ex) {
	    logger.logErrorMessage("Error while initializing keypad GUI.",
				   ex, "KeypadGUI(Controller, boolean)");
	}
	controller.addEventListener(this);
	addKeyListener(this);
	update();
    }

    //-------------------------------------------------------------------------
    // GUI-related methods.
    //-------------------------------------------------------------------------

    private void jbInit(boolean addMenuBar) throws Exception
    {
	jPanel1.setLayout(gridLayout1);
	gridLayout1.setRows(4);
	gridLayout1.setColumns(3);
	key1.setText("1");
	key1.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key1_actionPerformed(event);
		}
	    });
	keyHash.setText("#");
	keyHash.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    keyHash_actionPerformed(event);
		}
	    });
	key0.setText("0");
	key0.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key0_actionPerformed(event);
		}
	    });
	keyStar.setText("*");
	keyStar.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    keyStar_actionPerformed(event);
		}
	    });
	key9.setText("9");
	key9.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key9_actionPerformed(event);
		}
	    });
	key8.setText("8");
	key8.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key8_actionPerformed(event);
		}
	    });
	key7.setText("7");
	key7.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key7_actionPerformed(event);
		}
	    });
	key6.setText("6");
	key6.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key6_actionPerformed(event);
		}
	    });
	key5.setText("5");
	key5.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
        key5_actionPerformed(event);
		}
	    });
	key4.setText("4");
	key4.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key4_actionPerformed(event);
		}
	    });
	key3.setText("3");
	key3.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key3_actionPerformed(event);
		}
	    });
	key2.setText("2");
	key2.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    key2_actionPerformed(event);
		}
	    });
	hookButton.setPreferredSize(new Dimension(99,23));
	hookButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    hookButton_actionPerformed(event);
		}
	    });
	jPanel2.setLayout(borderLayout1);
	flashButton.setPreferredSize(new Dimension(99, 23));
	flashButton.setText("Flash");
	flashButton.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    flashButton_actionPerformed(event);
		}
	    });
	this.setTitle("Touchtone Keypad");
	this.addWindowListener(new java.awt.event.WindowAdapter() {
		public void windowClosing(WindowEvent event) {
		    this_windowClosing(event);
		}
	    });
	fileMenu.setText("File");
	debugMenuItem.setActionCommand("");
	debugMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    debugMenuItem_actionPerformed(event);
		}
	    });
	exitMenuItem.setText("Exit");
	exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    exitMenuItem_actionPerformed(event);
		}
	    });
	this.getContentPane().add(jPanel1, BorderLayout.CENTER);
	jPanel1.add(key1, null);
	jPanel1.add(key2, null);
	jPanel1.add(key3, null);
	jPanel1.add(key4, null);
	jPanel1.add(key5, null);
	jPanel1.add(key6, null);
	jPanel1.add(key7, null);
	jPanel1.add(key8, null);
	jPanel1.add(key9, null);
	jPanel1.add(keyStar, null);
	jPanel1.add(key0, null);
	jPanel1.add(keyHash, null);
	this.getContentPane().add(jPanel2, BorderLayout.SOUTH);
	jPanel2.add(hookButton, BorderLayout.WEST);
	jPanel2.add(flashButton, BorderLayout.EAST);
	fileMenuBar.add(fileMenu);
	fileMenu.add(debugMenuItem);
	fileMenu.add(exitMenuItem);

	if(addMenuBar)
	    setJMenuBar(fileMenuBar);
    }

    private void displayErrorMessage(String message)
    {
	JOptionPane.showMessageDialog(this, message, "Keypad Error",
				      JOptionPane.ERROR_MESSAGE);
    }
 
    private void update() 
    {
	updateFlashButton();
	updateHookButton();
	updateKeypad();
	updateDebugMenuItem();
    }
    
    private void updateFlashButton() 
    {
	flashButton.setEnabled(!controller.isOnHook());
    }
    
    private void updateHookButton() 
    {
	hookButton.setText(controller.isOnHook() ? "On Hook" : "Off Hook");
	hookButton.setForeground(controller.isOnHook() ? Color.black : Color.red);
    }
    
    private void updateKeypad() 
    {
	boolean isOffHook = !controller.isOnHook();
	key1.setEnabled(isOffHook);
	key2.setEnabled(isOffHook);
	key3.setEnabled(isOffHook);
	key4.setEnabled(isOffHook);
	key5.setEnabled(isOffHook);
	key6.setEnabled(isOffHook);
	key7.setEnabled(isOffHook);
	key8.setEnabled(isOffHook);
	key9.setEnabled(isOffHook);
	keyStar.setEnabled(isOffHook);
	key0.setEnabled(isOffHook);
	keyHash.setEnabled(isOffHook);
    }
    
    private void updateDebugMenuItem() 
    {
	debugMenuItem.setText(debugEnabled ? "Disable Debugging Output" : "Enable Debugging Output");
    }
        
    private void hookButton_actionPerformed(ActionEvent event) 
    {
	if(!controller.setOnHook(!controller.isOnHook()))
	    displayErrorMessage("Could not change \"on hook\" status.");
    }
    
    private void flashButton_actionPerformed(ActionEvent event)
    {
	if(!controller.flash())
	    displayErrorMessage("Error during \"flash\" operation.");
    }
    
    private void key1_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_1);
    }
    
    private void key2_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_2);
    }
    
    private void key3_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_3);
    }
    
    private void key4_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_4);
    }
    
    private void key5_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_5);
    }
    
    private void key6_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_6);
    }
    
    private void key7_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_7);
    }
    
    private void key8_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_8);
    }
    
    private void key9_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_9);
    }
    
    private void keyStar_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_STAR);
    }
    
    private void key0_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_0);
    }
    
    private void keyHash_actionPerformed(ActionEvent event) 
    {
	controller.touchtone(KeypadEvent.TOUCHTONE_HASH);
    }
    
    private void this_windowClosing(WindowEvent event) 
    {
	System.exit(0);
    }
    
    private void debugMenuItem_actionPerformed(ActionEvent event) 
    {
	controller.setDebugEnabled(!debugEnabled);
    } 
    
    private void exitMenuItem_actionPerformed(ActionEvent event) 
    {
	System.exit(0);
    }

    //-------------------------------------------------------------------------
    // Implementation of ControllerListener interface
    //-------------------------------------------------------------------------

    public void eventGenerated(UpdateEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: KeypadGUI handling " + event.toString());
	
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    update();
		}
	    });
    }
    
    public void eventGenerated(GUIEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: KeypadGUI handling " + event.toString());
	
	switch(event.getID()) {
	case GUIEvent.KEYPAD_INVISIBLE: case GUIEvent.KEYPAD_VISIBLE:
	    setVisible(controller.isKeypadVisible());
	    break;
	}
    }
    
    public void eventGenerated(KeypadEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: KeypadGUI handling " + event.toString());
	
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    update();
		}
	    });
    }
    
    public void eventGenerated(ModeEvent event) 
    {
	if (controller.isDebugEnabled())
	    logger.logMessage(Thread.currentThread().getName() + " thread: KeypadGUI handling " + event.toString());

	switch (event.getID()) {
	case ModeEvent.DEBUG_DISABLED:
	    debugEnabled = false;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			updateDebugMenuItem();
		    }
		});
	    break;
	case ModeEvent.DEBUG_ENABLED:
	    debugEnabled = true;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			updateDebugMenuItem();
		    }
		});
	    break;
	}
    }

    //-------------------------------------------------------------------------
    // Implementation of KeyListener interface
    //-------------------------------------------------------------------------

    public void keyTyped(KeyEvent event) 
    {
	if (debugEnabled)
	    logger.logMessage(Thread.currentThread().getName() + " thread: KeypadGUI handling " + event.toString());
	
	switch(event.getKeyChar()) {
	case '1':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key1.doClick();
		    }
		});
	    break;
	case '2':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key2.doClick();
		    }
		});
	    break;
	case '3':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key3.doClick();
		    }
		});
	    break;
	case '4':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key4.doClick();
		    }
		});
	    break;
	case '5':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key5.doClick();
		    }
		});
	    break;
	case '6':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key6.doClick();
		    }
		});
	    break;
	case '7':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key7.doClick();
		    }
		});
	    break;
	case '8':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key8.doClick();
		    }
		});
	    break;
	case '9':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key9.doClick();
		    }
		});
	    break;
	case '*':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			keyStar.doClick();
		    }
		});
	    break;
	case '0':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			key0.doClick();
		    }
		});
	    break;
	case '#':
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			keyHash.doClick();
		    }
		});
	    break;
	}
    }
    
    // Unused KeyListener methods.
    public void keyPressed(KeyEvent event) {}
    public void keyReleased(KeyEvent event) {}   
}
