/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
 * (c) Copyright 2000-2002 The MITRE Corporation (MITRE)
 *
 * Permission is hereby granted, without payment, to copy, use,
 * modify, display and distribute this software and its documentation,
 * if any, for any purpose, provided, first, that the US Government
 * and any of its agencies will not be charged any license fee and/or
 * royalties for the use of or access to said copyright software, and
 * provided further that the above copyright notice and the following
 * three paragraphs shall appear in all copies of this software,
 * including derivatives utilizing any portion of the copyright
 * software.  Use of this software constitutes acceptance of these
 * terms and conditions.
 *
 * IN NO EVENT SHALL MITRE BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT,SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF MITRE
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * MITRE SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTIES
 * INCLUDING,BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT.
 *
 * THE SOFTWARE IS PROVIDED "AS IS."  MITRE HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package org.mitre.audiogui.event;

import java.util.EventListener;
import org.mitre.audiogui.ControllerEvent;

/** 
 * Objects implementing the <code>ControllerListener</code> interface are 
 * expected to implement handler methods for events generated by the 
 * {@link org.mitre.audiogui.Controller} class. These events are subclasses of 
 * the abstract {@link ControllerEvent} class, which is sent to subscribers by
 * the <code>Controller.notifySubscribers</code> method. As a result, 
 * implementing objects should have a method taking a
 * <code>ControllerEvent</code> parameter and calling the appropriate subclass
 * handler method.  The {@link ControllerAdapter} class provides a reference 
 * method to do this, and also serves as an extensible abstract class for 
 * implementing a subset of the handlers listed.
 *
 *  @see org.mitre.audiogui.Controller#notifySubscribers(Object source, EventObject e)
 *  Controller.notifySubscribers
 *  @see ControllerAdapter 
 */

public interface ControllerListener extends EventListener 
{
    public void eventGenerated(UpdateEvent event);
    public void eventGenerated(GUIEvent event);
    public void eventGenerated(KeypadEvent event);
    public void eventGenerated(ModeEvent event);
}
