/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
 * (c) Copyright 2000-2002 The MITRE Corporation (MITRE)
 *
 * Permission is hereby granted, without payment, to copy, use,
 * modify, display and distribute this software and its documentation,
 * if any, for any purpose, provided, first, that the US Government
 * and any of its agencies will not be charged any license fee and/or
 * royalties for the use of or access to said copyright software, and
 * provided further that the above copyright notice and the following
 * three paragraphs shall appear in all copies of this software,
 * including derivatives utilizing any portion of the copyright
 * software.  Use of this software constitutes acceptance of these
 * terms and conditions.
 *
 * IN NO EVENT SHALL MITRE BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT,SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF MITRE
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * MITRE SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTIES
 * INCLUDING,BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT.
 *
 * THE SOFTWARE IS PROVIDED "AS IS."  MITRE HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

package org.mitre.audiogui.event;

import java.util.EventListener;
import java.util.EventObject;

/** 
 * The <code>ControllerGenerator</code> interface is used by
 * {@link org.mitre.audiogui.Controller} to fix its standard methods for 
 * managing subscribers and notifying them of events. 
 */

public interface EventGenerator 
{
    /** 
     * Adds the specified ControllerListener to the collection of current
     * listeners.
     *
     * @param l the listener to add
     */
    public void addEventListener(EventListener l);
    
    /** 
     * Removes the specified ControllerListener from the collection of current
     * listeners.
     *
     * @param l the listener to remove
     */
    public void removeEventListener(EventListener l);
    
    /** 
     * Provides a way for events to be generated and sent to subscribers.
     *
     * @param source the source of the event
     * @param event the event
     */
    public void notifySubscribers(Object source, EventObject event);    
}

