/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
  This file (c) Copyright 2002 The MITRE Corporation
  
  This file is part of the Communicator Audio Telephony Server
  (CATS). It is licensed under the conditions described in the file
  COPYING in the root directory of the CATS distribution.
*/

#include <stdlib.h>
#include <mdtm/mdtm.h>
#include <mdtm/mdtmmacros.h>
#include "cats_array_list.h"

static CatsArrayList CatsArrayList_destroy(CatsArrayList this);
static int CatsArrayList_add(CatsArrayList this, void *data);
static void *CatsArrayList_elementAt(CatsArrayList this, int index);
static void *CatsArrayList_firstElement(CatsArrayList this);
static void *CatsArrayList_remove(CatsArrayList this, int index);
static int CatsArrayList_size(CatsArrayList this);

/* PRIVATE METHODS */

static int CatsArrayList_ensureCapacity(CatsArrayList this, int minCapacity);

struct CatsArrayListPrivate_str {
  void **elementData;
  int capacity;
  int elementCount;
  int capacityIncrement;
};

CatsArrayList CatsArrayList_create(int initialCapacity, int capacityIncrement)
{
  CatsArrayList this=NULL;

  if (initialCapacity < 0)
    return NULL;

  if ((this =
       (CatsArrayList)
       calloc(1,sizeof(struct CatsArrayList_str))) != NULL) {
    CatsArrayListPrivate private;

    this->destroy = CatsArrayList_destroy;
    this->add = CatsArrayList_add;
    this->elementAt = CatsArrayList_elementAt;
    this->firstElement = CatsArrayList_firstElement;
    this->remove = CatsArrayList_remove;
    this->size = CatsArrayList_size;

    if ((private = this->private =
         (CatsArrayListPrivate)
	 calloc(1,sizeof(struct CatsArrayListPrivate_str)))
        == NULL)
      return(this->destroy(this));

    private->capacity = initialCapacity == 0 ? 10 : initialCapacity;
    if ((private->elementData = calloc(private->capacity, sizeof(void *)))
	== NULL)
      return(this->destroy(this));

    private->elementCount = 0;
    private->capacityIncrement = capacityIncrement;
  }

  return this;
}

static CatsArrayList CatsArrayList_destroy(CatsArrayList this)
{
  if (this->private != NULL) {
    if (this->private->elementData)
      free(this->private->elementData);
    free(this->private);
  }
  free(this);
  return NULL;
}

static int CatsArrayList_add(CatsArrayList this, void *data)
{
  int retval=MDTM_TRUE;

  if (CatsArrayList_ensureCapacity(this,this->private->elementCount + 1))
    this->private->elementData[this->private->elementCount++] = data;
  else
    retval = MDTM_FALSE;
  return retval;
}

static void *CatsArrayList_elementAt(CatsArrayList this, int index)
{
  void *element;
  element = index >= this->private->elementCount ? NULL :
    this->private->elementData[index];
  return element;
}

static void *CatsArrayList_firstElement(CatsArrayList this)
{
  void *firstElement;

  firstElement = this->private->elementCount == 0 ? NULL :
    this->private->elementData[0];
  return firstElement;
}

static void *CatsArrayList_remove(CatsArrayList this, int index)
{
  void *oldValue=NULL;

  if (index >= 0 && index < this->private->elementCount) {
    int numMoved;

    oldValue = this->private->elementData[index];
    numMoved = --this->private->elementCount-index;

    if (numMoved > 0) {
      char *cptr = (mdtm_vrchar *)this->private->elementData
	+ index*sizeof(void *);
      mdtm_vrchar_equ(cptr,cptr+sizeof(void *),numMoved*sizeof(void *));
    }
  }

  return oldValue;
}

/* PRIVATE METHODS */

static int CatsArrayList_size(CatsArrayList this)
{
  int retval;
  retval = this->private->elementCount;
  return retval;
}

static int CatsArrayList_ensureCapacity(CatsArrayList this, int minCapacity)
{
  int oldCapacity = this->private->capacity;
  if (minCapacity > oldCapacity) {
	int idx;
	void **tempElementData;
    /*void **elementData;*/
    int newCapacity = (this->private->capacityIncrement > 0) ?
      (oldCapacity + this->private->capacityIncrement) : (oldCapacity << 1);
    if (newCapacity < minCapacity)
      newCapacity = minCapacity;

	/* We do not fully trust using mdtm_vrchar_realloc here, so do reallocation by hand. */
	tempElementData = (void **) malloc(newCapacity*sizeof(void*));
	if (tempElementData == NULL) {
      return MDTM_FALSE;
	}

	for(idx=0; idx<oldCapacity; ++idx) {
		tempElementData[idx] = (this->private->elementData)[idx];
	}
	free(this->private->elementData);
	this->private->elementData = tempElementData;

	/*
    elementData =
      (void **) mdtm_vrchar_realloc((mdtm_vrchar *)this->private->elementData,
				    oldCapacity*sizeof(void *),
				    newCapacity*sizeof(void *));
    if (elementData == NULL)
      return MDTM_FALSE;

    this->private->elementData = elementData;
	*/

    this->private->capacity = newCapacity;
  }
  return MDTM_TRUE;
}
