/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
  This file (c) Copyright 2002 The MITRE Corporation
  
  This file is part of the Communicator Audio Telephony Server
  (CATS). It is licensed under the conditions described in the file
  COPYING in the root directory of the CATS distribution.
*/

#ifndef _CATS_CONTROLLER_H
#define _CATS_CONTROLLER_H

typedef struct CatsControllerPrivate_str *CatsControllerPrivate;
typedef struct CatsController_str *CatsController;

#include "cats_controller_parameters.h"
#include "cats_object.h"
#include "cats_event_listener.h"
#include "cats_server.h"
#include "cats_audio_format.h"
#include "cats_target_data_line.h"
#include "cats_source_data_line.h"
#include "cats_phone_line.h"
#include "cats_request.h"

struct CatsController_str {
  CatsObject object;
  CatsControllerPrivate private;
  CatsController (*destroy)(CatsController this);
  int (*open)(CatsController this);
  void (*close)(CatsController this);
  void (*shutDown)(CatsController this);

  /* Status query methods */

  int (*isBargeInEnabled)(CatsController this);
  int (*isDebugEnabled)(CatsController this);
  int (*isHalfDuplexEnabled)(CatsController this);
  int (*isPlayboxEnabled)(CatsController this);
  int (*useOutboundBrokers)(CatsController this);
  int (*isPTTEnabled)(CatsController this);
  int (*isStreamingEnabled)(CatsController this);
  int (*isCaptureLineOpen)(CatsController this);
  int (*isPlaybackLineOpen)(CatsController this);
  int (*isMuteActive)(CatsController this);
  int (*isOnHook)(CatsController this);
  int (*isRecordStartable)(CatsController this);
  int (*isRecordStarted)(CatsController this);
  int (*isResendable)(CatsController this);
  int (*isPlaybackStartable)(CatsController this);
  int (*isPlaybackStarted)(CatsController this);
  int (*isVADStartable)(CatsController this);
  int (*isVADStarted)(CatsController this);
  int (*isVoiceActivateable)(CatsController this);
  int (*isVoiceActive)(CatsController this);
  int (*isRequestRedundant)(CatsController this, void *caller,
			    void (*requesterCallback)(CatsRequest),
			    CatsObject requester);

  /* Parameter access methods */

  CatsAudioFormat (*getDefaultRecordAudioFormat)(CatsController this);
  CatsAudioFormat (*getDefaultPlaybackAudioFormat)(CatsController this);
  float (*getDefaultVadRaiseFactor)(CatsController this);
  float (*getDefaultVadLowerFactor)(CatsController this);
  CatsServer (*getServer)(CatsController this);
  void (*setADLine)(CatsController this, CatsTargetDataLine line);
  CatsTargetDataLine (*getADLine)(CatsController this);
  void (*setDALine)(CatsController this, CatsSourceDataLine line);
  CatsSourceDataLine (*getDALine)(CatsController this);
  void (*setPhoneLine)(CatsController this, CatsPhoneLine line);
  CatsPhoneLine (*getPhoneLine)(CatsController this);

  int (*ensureSwitchboard)(CatsController this);
  
  /* Mode setting methods */

  CatsRequest (*disableBargeIn)(CatsController this,
				void (*requesterCallback)(CatsRequest),
				CatsObject requester);
  CatsRequest (*enableBargeIn)(CatsController this,
			       void (*requesterCallback)(CatsRequest),
			       CatsObject requester);
  CatsRequest (*setBargeInEnabled)(CatsController this,
				   void (*requesterCallback)(CatsRequest),
				   CatsObject requester, int bargeInEnabled);
  CatsRequest (*disableHalfDuplex)(CatsController this,
				   void (*requesterCallback)(CatsRequest),
				   CatsObject requester);
  CatsRequest (*enableHalfDuplex)(CatsController this,
				  void (*requesterCallback)(CatsRequest),
				  CatsObject requester);
  CatsRequest (*setHalfDuplexEnabled)(CatsController this,
				      void (*requesterCallback)(CatsRequest),
				      CatsObject requester,
				      int halfDuplexEnabled);
  CatsRequest (*disablePlaybox)(CatsController this,
				void (*requesterCallback)(CatsRequest),
				CatsObject requester);
  CatsRequest (*enablePlaybox)(CatsController this,
			       void (*requesterCallback)(CatsRequest),
			       CatsObject requester);
  CatsRequest (*setPlayboxEnabled)(CatsController this,
				   void (*requesterCallback)(CatsRequest),
				   CatsObject requester,
				   int playboxEnabled);
  CatsRequest (*disablePTT)(CatsController this,
			    void (*requesterCallback)(CatsRequest),
			    CatsObject requester);
  CatsRequest (*enablePTT)(CatsController this,
			   void (*requesterCallback)(CatsRequest),
			   CatsObject requester);
  CatsRequest (*setPTTEnabled)(CatsController this,
			       void (*requesterCallback)(CatsRequest),
			       CatsObject requester, int pttEnabled);
  CatsRequest (*disableStreaming)(CatsController this,
				  void (*requesterCallback)(CatsRequest),
				  CatsObject requester);
  CatsRequest (*enableStreaming)(CatsController this,
				 void (*requesterCallback)(CatsRequest),
				 CatsObject requester);
  CatsRequest (*setStreamingEnabled)(CatsController this,
				     void (*requesterCallback)(CatsRequest),
				     CatsObject requester,
				     int streamingEnabled);
  CatsRequest (*disableDebug)(CatsController this,
			      void (*requesterCallback)(CatsRequest),
			      CatsObject requester);
  CatsRequest (*enableDebug)(CatsController this,
			     void (*requesterCallback)(CatsRequest),
			     CatsObject requester);
  CatsRequest (*setDebugEnabled)(CatsController this,
				 void (*requesterCallback)(CatsRequest),
				 CatsObject requester,
				 int streamingEnabled);

  /* Action request methods */

  CatsRequest (*offHook)(CatsController this,
			 void (*requesterCallback)(CatsRequest),
			 CatsObject requester);
  CatsRequest (*onHook)(CatsController this,
			void (*requesterCallback)(CatsRequest),
			CatsObject requester);
  CatsRequest (*touchtone)(CatsController this, int id,
			   void (*requesterCallback)(CatsRequest),
			   CatsObject requester);
  CatsRequest (*activateMute)(CatsController this,
			      void (*requesterCallback)(CatsRequest),
			      CatsObject requester);
  CatsRequest (*deactivateMute)(CatsController this,
				void (*requesterCallback)(CatsRequest),
				CatsObject requester);
  CatsRequest (*startPlayback)(CatsController this,
			       void (*requesterCallback)(CatsRequest),
			       CatsObject requester);
  CatsRequest (*stopPlayback)(CatsController this,
			      void (*requesterCallback)(CatsRequest),
			      CatsObject requester);
  CatsRequest (*bargeIn)(CatsController this,
			 void (*requesterCallback)(CatsRequest),
			 CatsObject requester);
  CatsRequest (*startRecord)(CatsController this,
			     void (*requesterCallback)(CatsRequest),
			     CatsObject requester);
  CatsRequest (*stopRecord)(CatsController this,
			    void (*requesterCallback)(CatsRequest),
			    CatsObject requester);
  CatsRequest (*resend)(CatsController this,
			void (*requesterCallback)(CatsRequest),
			CatsObject requester);
  CatsRequest (*startVAD)(CatsController this,
			  void (*requesterCallback)(CatsRequest),
			  CatsObject requester);
  CatsRequest (*stopVAD)(CatsController this,
			 void (*requesterCallback)(CatsRequest),
			 CatsObject requester);
  CatsRequest (*activateVoice)(CatsController this,
			       void (*requesterCallback)(CatsRequest),
			       CatsObject requester);
  CatsRequest (*deactivateVoice)(CatsController this,
				 void (*requesterCallback)(CatsRequest),
				 CatsObject requester);
  int (*addListener)(CatsController this, CatsEventListener l);
  void (*removeListener)(CatsController this, CatsEventListener l);
};

CatsController CatsController_create(CatsControllerParameters parameters,
				     CatsServer server);

#endif
