# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

CATS_ROOT = $(shell cd ..; /bin/pwd)

include $(CATS_ROOT)/cats_settings.make

# Modify this to point to the package root directory

ROOT_DIR = $(GC_HOME)
MITRE_ROOTDIR = $(GC_HOME)/contrib/MITRE
TEMPLATES = $(ROOT_DIR)/templates

# archos.make sets ARCH, OS, and ARCHOS for makefile conditionals.

include $(TEMPLATES)/archos.make

# Add any special CPP/LD flags here.

# We still want thread locking enabled, but we DON'T 
# want to allow thread compilation. We also want to
# require that the thread library be used.

CPPFLAGS += -I$(MDTM_INCLUDE) -I$(PA_INCLUDE) -DGAL_THREADS -D_REENTRANT
LDFLAGS  += -L$(MDTM_LIB) -L$(PA_LIB) -L$(CATS_LIBDIR)/$(ARCHOS)

# The order of objects for linking is as follows:
#  - object files (from SOURCES)
#  - SPECIAL_LIBS (defined below)
#    SPECIAL_LIBS are libraries with variants (_debug, _profile)
#    which are particular to the current executable.  If any of the
#    variants are non-standard, set SPECIAL_LIBS_D and/or SPECIAL_LIBS_P.
#  - LIBS (defined in <package>.make)
#    LIBS are libraries with variants which every executable in the
#    package requires.
#  - COMMON_LIBS (defined below and in <package>.make)
#    COMMON_LIBS are third party libraries without variants.
#  - SYSLIBS (defined in sysdep.make and <package>.make)
#    SYSLIBS are operating system libraries.

# Because I want this Makefile to always use the threaded versions
# of Communicator, I will use the NO_GALAXY hook in Galaxy.make, and
# then add all my own stuff.

NO_GALAXY = 1

include $(GC_HOME)/templates/shared_libs.make

COMMON_LIBS += -lmdtm -lportaudio -lpthread -lm
SPECIAL_LIBS += -lcats -lGalaxy_thread
LIBDEPS += $(GC_HOME)/lib/$(ARCHOS)/libGalaxy_thread$(LIBEXT)

ifdef SHARED_LIBS
  ifdef XLINKER_RTFLAG
    LDFLAGS += -Xlinker $(XLINKER_RTFLAG) -Xlinker $(GC_HOME)/lib/$(ARCHOS)
  endif
endif

# If the Galaxy Communicator library isn't compiled with 
# shared libs, we still want the LDFLAGS to have the PortAudio
# linker.

ifndef SHARED_LIBS
  ifdef XLINKER_RTFLAG
    LDFLAGS += -Xlinker $(XLINKER_RTFLAG) -Xlinker $(PA_LIB)
  endif
endif
