/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
  This file (c) Copyright 2002 The MITRE Corporation
  
  This file is part of the Communicator Audio Telephony Server
  (CATS). It is licensed under the conditions described in the file
  COPYING in the root directory of the CATS distribution.
*/

#include <stdlib.h>
#include <string.h>
#include <mdtm/mdtm.h>
#include <mdtm/mdtmmacros.h>
#include "cats_delay_line.h"
#include "cats_ring_buffer.h"
#include "cats_lock_object.h"

#define CATS_DEFAULT_BUFFER_SIZE 1024

static CatsDelayLine CatsDelayLine_destroy(CatsDelayLine this);
static int CatsDelayLine_readable(CatsDelayLine this);
static int CatsDelayLine_read(CatsDelayLine this, char *b, int off, int len);
static int CatsDelayLine_write(CatsDelayLine this, char *b, int off, int len);
static void CatsDelayLine_flush(CatsDelayLine this);
static int CatsDelayLine_getBufferSize(CatsDelayLine this);

struct CatsDelayLinePrivate_str {
  CatsRingBuffer buffer;
};

CatsDelayLine CatsDelayLine_create(int bufferSize)
{
  CatsDelayLine this=NULL;

  if ((this = (CatsDelayLine) calloc(1,sizeof(struct CatsDelayLine_str)))
      != NULL) {
    CatsDelayLinePrivate private=NULL;
    char zero=0;

    this->destroy = CatsDelayLine_destroy;
    this->readable = CatsDelayLine_readable;
    this->read = CatsDelayLine_read;
    this->write = CatsDelayLine_write;
    this->flush = CatsDelayLine_flush;
    this->getBufferSize = CatsDelayLine_getBufferSize;

    if ((private = this->private =
         (CatsDelayLinePrivate)
	 calloc(1,sizeof(struct CatsDelayLinePrivate_str))) == NULL)
      return(this->destroy(this));

    if ((private->buffer = CatsRingBuffer_create(bufferSize)) == NULL)
      return(this->destroy(this));

    while(private->buffer->writeable(private->buffer) > 0)
      private->buffer->write(private->buffer,&zero,0,1);
	     
  }
  return this;
}

static CatsDelayLine CatsDelayLine_destroy(CatsDelayLine this)
{
  if (this->private != NULL) {
    CatsDelayLinePrivate private=this->private;
    if (private->buffer != NULL)
      private->buffer->destroy(private->buffer);
    free(this->private);
  }
  free(this);
  return NULL;
}

static int CatsDelayLine_readable(CatsDelayLine this)
{
  return this->private->buffer->readable(this->private->buffer);
}

static int CatsDelayLine_read(CatsDelayLine this, char *b, int off, int len)
{
  return this->private->buffer->read(this->private->buffer,b,off,len);
}

static int CatsDelayLine_write(CatsDelayLine this, char *b, int off, int len)
{
  CatsRingBuffer buffer=this->private->buffer;
  int bytesToWrite,bytesWritten=0;
  char scratchByte;

  if ((bytesToWrite = MDTM_MIN(buffer->writeable(buffer),len)) > 0 &&
      (bytesWritten = buffer->write(buffer,b,off,bytesToWrite)) == -1)
    return -1;

  len -= bytesWritten;
  off += bytesWritten;

  while (len > 0) {
    buffer->read(buffer,&scratchByte,0,1);
    if (buffer->write(buffer,b,off++,1) == -1)
      return -1;
    len--;
  }

  return len;
}

static void CatsDelayLine_flush(CatsDelayLine this)
{
  this->private->buffer->flush(this->private->buffer);
}

static int CatsDelayLine_getBufferSize(CatsDelayLine this)
{
  return this->private->buffer->getBufferSize(this->private->buffer);
}
