/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*
  This file (c) Copyright 2002 The MITRE Corporation
  
  This file is part of the Communicator Audio Telephony Server
  (CATS). It is licensed under the conditions described in the file
  COPYING in the root directory of the CATS distribution.
*/

#ifndef _CATS_REQUEST_H
#define _CATS_REQUEST_H

/* Request status codes */

#define CATS_REQUEST_INACTIVE (0)
#define CATS_REQUEST_SUCCESS (1)
#define CATS_REQUEST_FAILURE (2)
#define CATS_REQUEST_ABORTED (3)
#define CATS_REQUEST_PENDING (4)

typedef struct CatsRequest_str *CatsRequest;

#include "cats_vector.h"
#include "cats_request_queue.h"
#include "cats_object.h"

/* Constructor */

CatsRequest
CatsRequest_create(void *caller, void (*monitorCallback)(CatsRequest),
		   void *monitor, void (*requesterCallback)(CatsRequest),
		   CatsObject requester, void *requestData);

/* EBG: The request object holds information needed to asynchronously
   fire and monitor completion of a request to an object, through the
   monitorCallback function, and to asynchronously invoke a callback
   function in a requesting object, through the requesterCallback
   function. When the request's status field is CATS_REQUEST_INACTIVE,
   the monitorCallback function is expected to perform whatever action
   is required in the monitor object to invoke a request, and is
   expected to set the status field to CATS_REQUEST_SUCCESS if the
   monitor callback detects successful completion of the task,
   CATS_REQUEST_FAILURE if the request cannot be satisfied, or
   CATS_REQUEST_PENDING if the task has yet to complete.  The
   requesterCallback is expected to check for success or failure in
   the context of the requesting object and act accordingly. */

struct CatsRequest_str {
  CatsObject object;

  void *caller;
  void *monitor;
  CatsObject requester;
  void *requestData;
  CatsRequest parent;
  CatsVector children;
  CatsRequestQueue requestQueue;
  int status;
  CatsRequest (*destroy)(CatsRequest this);
  int (*addChild)(CatsRequest this, CatsRequest request);
  void (*removeChild)(CatsRequest this, CatsRequest request);
  int (*equals)(CatsRequest this, CatsRequest request);
  int (*isRecursive)(CatsRequest this);
  void (*monitorCallback)(CatsRequest request);
  void (*requesterCallback)(CatsRequest request);
};

#endif
