/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

#include <stdlib.h>
#include <mdtm/mdtm.h>
#include <mdtm/mdtmmacros.h>
#include "pa_mixer.h"
#include "cats_audio_format.h"

#define PARSESTRING "{[--buflen|-b%d][--duration|-d%d][--latency|-l %d][--little_endian|-le]}"

int main(int argc, char *argv[]);

int main(int argc, char *argv[])
{
  PaMixer paMixer;
  CatsTargetDataLine targetLine;
  CatsSourceDataLine sourceLine;
  CatsAudioFormat format;

  mdtm_vrchar *buffer;
  int duration=5;
  int latency=250;
  int bufferLength=0;
  int littleEndian=MDTM_FALSE;
  int bytesPerSecond;
  int bytesRead;
  int lineBufferLength;

  mdtm_clparse(argc,argv,PARSESTRING,4,&bufferLength,&duration,&latency,
	       &littleEndian);

  paMixer = PaMixer_create(MDTM_FALSE);
  targetLine = paMixer->getTargetDataLine(paMixer);
  sourceLine = paMixer->getSourceDataLine(paMixer);

  format =
    CatsAudioFormat_create(CATS_ENCODING_PCM_SIGNED,8000.,16,1,2,8000.,
			   !littleEndian);
  bytesPerSecond = format->getFrameRate(format)*format->getFrameSize(format);
  printf("bytesPerSecond = %d\n",bytesPerSecond);
  lineBufferLength = (latency*bytesPerSecond)/1000;

  targetLine->open(targetLine,format,lineBufferLength);
  sourceLine->open(sourceLine,format,lineBufferLength);

  if (bufferLength == 0) {
    bufferLength = duration*bytesPerSecond;
    buffer = mdtm_vrchar_malloc(bufferLength);
    targetLine->start(targetLine);
    bytesRead = targetLine->read(targetLine,buffer,0,bufferLength);
    targetLine->stop(targetLine);
    targetLine->close(targetLine);
    sourceLine->start(sourceLine);
    sourceLine->write(sourceLine,buffer,0,bufferLength);
    sourceLine->drain(sourceLine);
    sourceLine->stop(sourceLine);
    sourceLine->close(sourceLine);
  } else {
    int bytesToRead = duration*bytesPerSecond;
    int totalBytesRead=0;

    buffer = mdtm_vrchar_malloc(bufferLength);

    targetLine->start(targetLine);
    sourceLine->start(sourceLine);

    while(totalBytesRead < bytesToRead) {
      totalBytesRead += bytesRead =
	targetLine->read(targetLine,buffer,0,bufferLength);
      sourceLine->write(sourceLine,buffer,0,bytesRead);
    }

    targetLine->stop(targetLine);
    sourceLine->drain(sourceLine);
    sourceLine->stop(sourceLine);
    targetLine->close(targetLine);
    sourceLine->close(sourceLine);
  }

  mdtm_vrchar_free(buffer);
  format->destroy(format);
  paMixer->destroy(paMixer);
  return 0;
}
