# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import os, sys, string, math, random

# This only works in 3.0.

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy, TkMGalaxy, GalaxyIO, SLSUtil

# Windowing.

import Tkinter
from Tkinter import *
from TkMGalaxy import HEADERFONT, CONTENTSFONT, MenuBarEntry, ScrollPane

EffectsHelp = """
           avg [ -l | -r | -f | -b | n,n,...,n ]
           band [ -n ] center [ width ]
           bandpass frequency bandwidth
           bandreject frequency bandwidth
           chorus gain-in gain out delay decay speed depth
                  -s | -t [ delay decay speed depth -s | -t ]
           compand attack1,decay1[,attack2,decay2...]
                   in-dB1,out-dB1[,in-dB2,out-dB2...]
                   [ gain [ initial-volume [ delay ] ] ]
           copy
           dcshift shift [ limitergain ]
           deemph
           earwax
           echo gain-in gain-out delay decay [ delay decay ... ]
           echos gain-in gain-out delay decay [ delay decay ... ]
           fade [ type ] fade-in-length
                [ stop-time [ fade-out-length ] ]
           filter [ low ]-[ high ] [ window-len [ beta ]]
           flanger gain-in gain-out delay decay speed < -s | -t >
           highp frequency
           highpass frequency
           lowp frequency
           lowpass frequency
           map
           mask
           pan direction
           phaser gain-in gain-out delay decay speed < -s | -t >
           pick [ -1 | -2 | -3 | -4 | -l | -r ]
           pitch shift [ width interpole fade ]
           polyphase [ -w < nut / ham > ]
                     [  -width < long / short / # > ]
                     [ -cutoff # ]
           rate
           resample [ -qs | -q | -ql ] [ rolloff [ beta ] ]
           reverb gain-out reverb-time delay [ delay ... ]
           reverse
           silence above_periods [ duration threshold[ d | % ]
                   [ below_periods duration
                     threshold[ d | % ]]
           speed [ -c ] factor
           split
           stat [ -s n ] [ -rms ] [ -v ] [ -d ]
           stretch [ factor [ window fade shift fading ]
           swap [ 1 2 | 1 2 3 4 ]
           synth [ length ] type mix [ freq [ -freq2 ]
                 [ off ] [ ph ] [ p1 ] [ p2 ] [ p3 ]
           trim start [ length ]
           vibro speed [ depth ]
           vol gain [ type [ limitergain ] ]
"""

EffectsHelpHeight = 0
EffectsHelpWidth = 0
EffectsNames = []
first_time = 1
EffectsSpace = 0

for i in string.split(EffectsHelp, "\n"):
    EffectsHelpHeight = EffectsHelpHeight + 1
    b = string.split(i)
    if b:
        if first_time:
            EffectsSpace = string.find(i, b[0])
            EffectsNames.append(b[0])
            first_time = 0
        elif string.find(i, b[0]) == EffectsSpace:
            EffectsNames.append(b[0])
    if len(i) > EffectsHelpWidth:
        EffectsHelpWidth = len(i)

class HelpWindow(Tkinter.Frame):
    def __init__(self):
        Frame.__init__(self, Toplevel())
        self.master.title("Effects help")
        help_window = Text(self, relief = 'flat',
                           height = EffectsHelpHeight,
                           width = EffectsHelpWidth,
                           font = CONTENTSFONT)
        help_window['state']='normal'
	help_window.insert('end', EffectsHelp)
	help_window['state']='disabled'
        help_window.pack(side = 'top', anchor = 'w',
                         fill = 'both', expand = 1)
        dismiss_button = Button(self, text = "Dismiss",
                                relief = 'ridge',
                                font = HEADERFONT,
                                command = self.master.destroy)
        dismiss_button.pack(side = 'top', anchor = 'w',
                            fill = 'x')
        self.pack()

# Things we can set:
# :effects - list of lists
# :volume - float
# :output_sample_rate
# :output_data_encoding (PCM_SIGNED, PCM_UNSIGNED, ULAW, ALAW,
#                        FLOATING_POINT, ADPCM, IMA_ADPCM, GSM)
# :output_channels (1, 2, 4)
# :output_data_size (8, 16, 32)
# :output_filename

DataEncodings = ["PCM_SIGNED", "PCM_UNSIGNED", "ULAW",
                 "ALAW", "FLOATING_POINT", "ADPCM", "IMA_ADPCM", "GSM"]
OutputChannels = [1, 2, 4]
OutputDataSize = [8, 16, 32]

# We need a setting line class. There's a delete
# key, a menu or label, an optional argument type-in.

class SettingLine(Frame):

    def __init__(self, master, parent, label, prefix, frame_key,
                 menu_options = None, need_type_in = 0):

        Frame.__init__(self, master, relief = "ridge",
                       borderwidth = 2)
        self.parent = parent
        self.frame_key = frame_key
        self.prefix = prefix
        if menu_options is not None:
            self.label = label + "..."
        else:
            self.label = label + ":"
        
        # Left to right, there's the delete button, the menu,
        # and the arguments window.
        delete_button = Button(self, text = "Delete",
                               relief = 'groove',
                               font = HEADERFONT)
        delete_button['command'] = lambda s = self: s.parent.DeleteSetting(s, s.prefix)
        
        delete_button.pack(side = 'left', anchor = 'center')

        if menu_options is None:
            # Just put in a label.
            l = Label(self, text=self.label, font= CONTENTSFONT)
            l.pack(side = 'left', anchor = 'center')
            self.mb = None
        else:
        
            # Next, we construct the menu.
        
            self.mb = Menubutton(self, font = CONTENTSFONT,
                                 text = self.label)
            menu = Menu(self.mb, font= CONTENTSFONT, tearoff=0)

            self.mb['menu'] = menu

            for e in menu_options:
                e = str(e)
                menu.add('command', label = e,
                         command = lambda s = self, l = e: s.ChooseSetting(l))
        
            self.mb.pack(side = 'left', anchor = 'center')
        if need_type_in:
            # Next, the text widget.
            self.args = Tkinter.StringVar()
            entry = Entry(self, borderwidth = 2,
                          relief = 'sunken',
                          font = CONTENTSFONT,
                          width = 40,
                          textvariable = self.args)
            entry.pack(side = 'left', anchor = 'center',
                       fill = 'x')            
        else:
            self.args = None
        self.pack(side = 'top', anchor = 'w',
                  fill = 'x')
        self.AddSetting()
        self.val = None
        
    def AddSetting(self):
        self.parent.AddSetting(self, self.prefix)

    def ChooseSetting(self, val):
        self.val = val
        self.mb['text'] = self.prefix + ": " + val

    def ReportSetting(self, frame):
        if self.args is not None:
            args = self.args.get()
        else:
            args = ""
        if self.mb is None:
            self.AddValueToFrame(frame, self.frame_key, "", args)
        elif self.val is not None:
            self.AddValueToFrame(frame, self.frame_key, self.val, args)

    def AddValueToFrame(self, f, frame_key, label, args):
        f[frame_key] = label + args

class EffectLine(SettingLine):
    def __init__(self, master, parent):
        SettingLine.__init__(self, master, parent, "Choose effect", "Effect",
                             ":effects", menu_options = EffectsNames,
                             need_type_in = 1)

    def AddSetting(self):
        self.parent.AddSetting(self, self.prefix, disable = 0)

    def AddValueToFrame(self, f, frame_key, label, args):
        if f.has_key(frame_key):
            f[frame_key].append([label] + string.split(args))
        else:
            f[frame_key] = [[label] + string.split(args)]

class IntSettingLine(SettingLine):
    def AddValueToFrame(self, f, frame_key, label, args):
        # Either one or the other will be set.
        s = label + args
        try:
            f[frame_key] = string.atoi(s)
        except ValueError:
            print "Skipping non-integer value", s, "for", frame_key
        
class FloatSettingLine(SettingLine):
    def AddValueToFrame(self, f, frame_key, label, args):
        # Either one or the other will be set.
        s = label + args
        try:
            f[frame_key] = string.atof(s)
        except ValueError:
            print "Skipping non-float value", s, "for", frame_key
        
class SoxGUI(TkMGalaxy.CommunicatorInterface):
    
    def __init__(self, master = None, env = None):
	TkMGalaxy.CommunicatorInterface.__init__(self, master, env)
	self.master.title("SoX GUI")
	self['borderwidth'] = 2
	self.pack(fill = 'both', expand = 1)
	self.topmenubar = Tkinter.Frame(self, relief = 'groove',
					borderwidth = 2)
	self.topmenubar.pack(side = 'top', fill = 'x', anchor = 'w')
	b1 = MenuBarEntry(self.topmenubar, "File",
			  {"Quit": ('command', self.Quit)})
	b1.pack(side = 'left')
        b2 = MenuBarEntry(self.topmenubar, "Add...",
                          {"Effect": ('command', lambda s = self: EffectLine(s.effects_pane, s)),
                           "Volume": ('command', lambda s = self: FloatSettingLine(s.settings_pane, s, "Set volume", "Volume", ":volume", need_type_in = 1)),
                           "Sample rate": ('command', lambda s = self: IntSettingLine(s.settings_pane, s, "Set sample rate", "Sample rate", ":output_sample_rate", need_type_in = 1)),
                           "Date encoding": ('command', lambda s = self: SettingLine(s.settings_pane, s, "Choose data encoding", "Date encoding", ":output_data_encoding", menu_options = DataEncodings)),
                           "Channels": ('command', lambda s = self: IntSettingLine(s.settings_pane, s, "Choose channels", "Channels", ":output_channels", menu_options = OutputChannels)),
                           "Data size": ('command', lambda s = self: IntSettingLine(s.settings_pane, s, "Choose data size", "Data size", ":output_data_size", menu_options = OutputDataSize)),
                           "Input file": ('command', lambda s = self: SettingLine(s.settings_pane, s, "Choose input file", "Input file", ":input_filename", need_type_in = 1)),
                           "Output file": ('command', lambda s = self: SettingLine(s.settings_pane, s, "Choose output file", "Output file", ":output_filename", need_type_in = 1))})

        b2.pack(side = 'left')
        self.add_mb = b2
        b3 = MenuBarEntry(self.topmenubar, "Help",
			  {"Effects help": ('command', HelpWindow) } )
	b3.pack(side = 'left')
        
        # The effects are in order. The settings list
        # will start out empty.
        self.settings = []
        self.effects_pane = Frame(self, relief = "flat",
                                  borderwidth = 0)
        self.effects_pane.pack(side = 'top', fill = 'x', anchor = 'w')

        self.settings_pane = Frame(self, relief = "flat",
                                  borderwidth = 0)
        self.settings_pane.pack(side = 'top', fill = 'x', anchor = 'w')
        
        send_button = Button(self, text = "Send settings",
                             relief = 'groove',
                             font = HEADERFONT,
                             command = self.SendSettings)
        send_button.pack(side = 'top', anchor = 'w',
                         fill = 'x')

    def SendSettings(self):
        # Now, we collect the settings.
        e = []
        f = Galaxy.Frame(name = "FromSoxGUI")        
        for eff in self.settings:
            eff.ReportSetting(f)
        self.env.WriteFrame(f)

    def AddSetting(self, setting, menu_entry, disable = 1):
        self.settings.append(setting)
        # Disable the menu entry.
        if disable:
            self.add_mb.menu.entryconfigure(menu_entry, state = "disabled")

    def DeleteSetting(self, setting, menu_entry):
        self.add_mb.menu.entryconfigure(menu_entry, state = "normal")
        self.settings.remove(setting)
        setting.pack_forget()
    
# Communicator server.

OAS = [("-cats_compatibility", "use only CATS-compatible formats")]

class SoxGUIServer(TkMGalaxy.Server):
    def CheckUsage(self, oas_list, args):
        data, out_args = TkMGalaxy.Server.CheckUsage(self, OAS + oas_list,
                                                     args)
        if data.has_key("-cats_compatibility"):
            global DataEncodings
            DataEncodings = ["PCM_SIGNED", "PCM_UNSIGNED", "ULAW", "ALAW"]
            del data["-cats_compatibility"]
        return data, out_args        
    
def main():
    i = SoxGUIServer(sys.argv, "sox_gui", default_port = 9777,
                     gui_class = SoxGUI)
    i.RunServer()

main()
