# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import os, sys, string, math, random

# This only works in 3.0.

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy, TkMGalaxy, GalaxyIO, SLSUtil

# Windowing.

import Tkinter

from TkMGalaxy import HEADERFONT, CONTENTSFONT, MenuBarEntry, ScrollPane

class IOPodium(TkMGalaxy.CommunicatorInterface):
    
    def __init__(self, master = None, env = None):
	TkMGalaxy.CommunicatorInterface.__init__(self, master, env)
	self.master.title("I/O Traffic")
	self['borderwidth'] = 2
	self.pack(fill = 'both', expand = 1)
	self.topmenubar = Tkinter.Frame(self, relief = 'groove',
					borderwidth = 2)
	self.topmenubar.pack(side = 'top', fill = 'x', anchor = 'w')
	b1 = MenuBarEntry(self.topmenubar, "File",
			  {"Quit": ('command', self.Quit),
			   "Show/hide history":
			   ('command', self.ToggleHistory),
			   "Clear history":
			   ('command', self.ClearHistory)})
	b1.pack(side = 'left')

	# Here's the palette for the buttons.
	palette = Tkinter.Frame(self, relief = "groove",
				borderwidth = 2)
	palette.pack(side = 'top', anchor = 'w',
		     fill = 'both', expand = 1)
	l1 = Tkinter.Label(palette, text = "In:", font = HEADERFONT)
	l2 = Tkinter.Label(palette, text = "Out:", font = HEADERFONT)
	# l1.grid(row = 0, column = 0, sticky = 'news')
	# l2.grid(row = 1, column = 0, sticky = 'news')
	l1.grid(row = 0, column = 0)
	l2.grid(row = 1, column = 0)
	# l1.pack(side = 'left')
	self.history = ScrollPane(self, height = 10)
	self.history_visible = 0

	self.invar = Tkinter.StringVar()
	self.outvar = Tkinter.StringVar()
        self.input_id = 0
        self.output_id = 0
	self.in_entry = Tkinter.Entry(palette, borderwidth = 2,
				      relief = 'sunken', font = CONTENTSFONT,
				      width = 40, textvariable = self.invar)
	self.out_entry = Tkinter.Entry(palette, borderwidth = 2,
				       relief = 'sunken', font = CONTENTSFONT,
				       width = 40, textvariable = self.outvar)
	self.in_entry.bind('<KeyPress-Return>',
			   lambda event, s = self: s.SendInput())
	self.out_entry.bind('<KeyPress-Return>',
			    lambda event, s = self: s.SendOutput())

	self.in_entry.grid(row = 0, column = 1, sticky = 'nesw')
	self.out_entry.grid(row = 1, column = 1, sticky = 'nesw')
	
	palette.grid_columnconfigure(1, weight = 1)
	palette.grid_rowconfigure(0, weight = 1)
	palette.grid_rowconfigure(1, weight = 1)

    def ToggleHistory(self):
	if self.history_visible:
	    self.history.pack_forget()
	    self.history_visible = 0
	else:
	    self.history.pack(side = 'top', anchor = 'w',		  
			      fill = 'both', expand = 1)
	    self.history_visible = 1

    def ClearHistory(self):
	self.history.Clear()
	
    def SendInput(self):
	s = self.invar.get()
	if s:
	    f = Galaxy.Frame(name = "FromIOPodium",
			     contents = {":input_string": s,
                                         ":input_id": self.input_id})
            self.env.WriteFrame(f)
            self.input_id = self.input_id + 1

    def ShowInput(self, s):
	self.invar.set(s)
	self.history.Write("U: " + string.strip(s) + "\n")

    def ClearInput(self):
	self.invar.set("")

    def SendOutput(self):
	s = self.outvar.get()
	if s:
	    f = Galaxy.Frame(name = "FromIOPodium",
			     contents = {":output_string": s,
                                         ":output_id": self.output_id})
	    self.env.WriteFrame(f)
            self.output_id = self.output_id + 1

    def ShowOutput(self, s):
	self.outvar.set(s)
	self.history.Write("S: " + string.strip(s) + "\n")

    def ClearOutput(self):
	self.outvar.set("")

# Communicator server.

# This server should accept and issue the message set for the speech monitor.

class IOEnvironment(GalaxyIO.CallEnvironment):

    def ShowInputString(self, dict):
        try:
            self.conn.gui.ShowInput(dict[":input_string"])
            # Reset the input ID if provided.
            if dict.has_key(":input_id"):
                self.conn.gui.input_id = dict[":input_id"] + 1
            if dict.has_key(":clear_output"):
                self.conn.gui.ClearOutput()
        except:
            pass

    def ShowOutputString(self, dict):
        try:
            self.conn.gui.ShowOutput(dict[":output_string"])
            # Reset the output ID if provided.
            if dict.has_key(":output_id"):
                self.conn.gui.output_id = dict[":output_id"] + 1
            if dict.has_key(":clear_input"):
                self.conn.gui.ClearInput()
        except:
            pass

    def Welcome(self, dict):
        self.conn.gui.input_id = 0
        self.conn.gui.output_id = 0
	self.conn.gui.ClearInput()
	self.conn.gui.ClearOutput()
    
def main():
    i = TkMGalaxy.Server(sys.argv, "io_podium", default_port = 13050,
			 env_class = IOEnvironment,
			 gui_class = IOPodium)
    i.AddDispatchFunction("reinitialize", IOEnvironment.Welcome)
    i.AddDispatchFunction("show_input_string", IOEnvironment.ShowInputString)
    i.AddDispatchFunction("show_output_string", IOEnvironment.ShowOutputString)
    i.RunServer()

main()
