# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import os, sys, string

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy, TkMGalaxy, GalaxyIO, SLSUtil, Tkinter

OAS = [("-prompt <str>", "use this prompt",
	[type("")], ["Type a sentence: "]),
       ("-prog_name <str>", "name of program to send messages to",
	[type("")], ["rec"])]

class InputServer(TkMGalaxy.Server):

    def CheckUsage(self, oas_list, args):
	data, out_args = TkMGalaxy.Server.CheckUsage(self, OAS + oas_list, args)
	self.prog_name = data["-prog_name"][0]
	# Ignore the prompt for GUI.
	del data["-prompt"]
	del data["-prog_name"]
	return data, out_args


########################################
#
# main Communicator interface
#
########################################

from TkMGalaxy import MenuBarEntry, ScrollPane, LabeledEntry

class UIInterface(TkMGalaxy.CommunicatorInterface):
    def __init__(self, master = None, env = None):
	TkMGalaxy.CommunicatorInterface.__init__(self, master, env)
	self.master.title("Communicator GUI")
	self['borderwidth'] = 2
	self.pack(fill = 'both', expand = 1)
	self.topmenubar = Tkinter.Frame(self, relief = 'groove', borderwidth = 2)
	self.topmenubar.pack(side = 'top', fill = 'x', anchor = 'w')
	b1 = MenuBarEntry(self.topmenubar, "File",
			  {"Quit": ('command', self.Quit)})
	b1.pack(side = 'left')
	self.history = ScrollPane(self,
				  label = "Monitor",
				  width = 40,
				  height = 10)
	self.history.pack(side = 'top', anchor = 'w',		  
			  fill = 'both', expand = 1)
	self.io = ScrollPane(self,
			     label = "Dialogue history",
			     width = 40,
			     height = 10)
	self.io.pack(side = 'top', anchor = 'w',
		     fill = 'both', expand = 1)
	self.typed_input = LabeledEntry(self, "Input",
					return_cmd = self.InputCallback)
	self.typed_input.pack(side = 'top', anchor = 'w',
			      fill = 'x', expand = 0)

    def InputCallback(self, str):
	self.typed_input.SetValue("")
	self.env.InputCallback(str)

    def Quit(self, do_conn = 1):
	# Remove yourself from the parent's ostream.
	if self.env:
	    self.env.conn.ostream.gui = None
	TkMGalaxy.CommunicatorInterface.Quit(self, do_conn)

# We construct a toplevel Communicator interface for
# each connection. Here, we want to redirect I/O to the GUI.

class Connection(TkMGalaxy.Connection):
    def __init__(self, server, c_conn):
	TkMGalaxy.Connection.__init__(self, server, c_conn)
	self.ostream = GuiOStream(self.gui)
	self.counter = 0

class GuiOStream(SLSUtil.OStream):
    def __init__(self, gui, *args):
	self.gui = gui
	apply(SLSUtil.OStream.__init__, (self,) + args)
    def _print(self, msg, level, fp = sys.stdout):
	if self.gui:
	    self.gui.history.Write(msg + "\n")
	else:
	    SLSUtil.OStream._print(self, msg, level, fp)

class InputEnvironment(GalaxyIO.CallEnvironment):
    
    def ReportInput(self, msg):
	if self.conn.gui:
	    # Write a boundary to the history
	    self.conn.gui.history.Write("(%d ends) =========================\n" % self.conn.counter)	    
	    self.conn.gui.io.Write("[Input (%d)] %s\n" % (self.conn.counter, msg))
	    self.conn.counter = self.conn.counter + 1
	else:
	    self.ostream._print(self, "[Input] " + msg)
    def ReportOutput(self, msg):
	if self.conn.gui:
	    # Write a boundary to the history
	    self.conn.gui.history.Write("(%d ends) =========================\n" % self.conn.counter)
	    self.conn.gui.io.Write("[Output (%d)] %s\n" % (self.conn.counter, msg))
	    self.conn.counter = self.conn.counter + 1
	else:
	    self.ostream._print(self, "[Output] " + msg)

    def Welcome(self, dict):
	if dict.has_key(":greeting"):
	    self.ReportOutput(dict[":greeting"])
	self.conn.utterance_id = 0
	return None

    def Present(self, dict):
	try:
	    self.conn.utterance_id = dict[":utterance_id"]
	except KeyError:
	    self.conn.utterance_id = 0
	if dict.has_key(":paraphrase_string"):
	    self.ReportInput(dict[":paraphrase_string"])
	if dict.has_key(":reply_string"):
	    self.ReportOutput(dict[":reply_string"])
	if dict.has_key(":object"):
	    self.ReportOutput("\n" + dict[":object"].PPrint())
	return dict

    def InputCallback(self, str):
	f = Galaxy.Frame(self.server.prog_name, Galaxy.GAL_CLAUSE,
			 {":input_string": str,
			  ":hyp_id": 0,
			  ":utterance_id": self.conn.utterance_id,
			  ":mode": "typed"})
	self.WriteFrame(f)
	return None

def main():
    i = InputServer(sys.argv, "ui", default_port = 15010,
		    gui_class = UIInterface,
		    conn_class = Connection,
		    env_class = InputEnvironment)
    i.AddDispatchFunction("reinitialize", InputEnvironment.Welcome)
    i.AddDispatchFunction("enable_input", InputEnvironment.Present)
    i.RunServer()

main()
