# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import os, sys, string, math, random

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy, MGalaxy, GalaxyIO

OAS = [("-prompt <str>", "use this prompt",
	[type("")], ["Type a sentence: "]),
       ("-prog_name <str>", "name of program to send messages to",
	[type("")], ["rec"])]

class TextStdinPoll(MGalaxy.StdinConnection):

    def __init__(self, *args):
	apply(MGalaxy.StdinConnection.__init__, (self,) + args)
	
    def CreateFrame(self, str):	
	return Galaxy.Frame(self.prog_name, Galaxy.GAL_CLAUSE,
			    {":input_string": str,
			     ":utterance_id": self.utterance_id,
			     ":hyp_id": 0,
			     ":mode": "typed"})

class InputServer(GalaxyIO.Server):

    def CheckUsage(self, oas_list, args):
	data, out_args = GalaxyIO.Server.CheckUsage(self, OAS + oas_list, args)
	self.prompt = data["-prompt"][0]
	self.prog_name = data["-prog_name"][0]
	del data["-prompt"]
	del data["-prog_name"]
	return data, out_args

class InputEnvironment(GalaxyIO.CallEnvironment):

    def Welcome(self, dict):
	if dict.has_key(":greeting"):
	    self.ostream.write("[Greeting] " + dict[":greeting"])
	if not hasattr(self, "stdin_poll"):
	    self.conn.stdin_poll = TextStdinPoll(self, self.server.prompt)
	self.conn.stdin_poll.Activate()
	self.conn.stdin_poll.utterance_id = 0
	self.conn.stdin_poll.prog_name = self.server.prog_name
	return None

    def Present(self, dict):
	try:
	    self.conn.stdin_poll.utterance_id = dict[":utterance_id"]
	except KeyError:
	    self.conn.stdin_poll.utterance_id = 0
	if dict.has_key(":paraphrase_string"):
	    self.ostream.write("[Input] " + dict[":paraphrase_string"])
	if dict.has_key(":reply_string"):
	    self.ostream.write("[Output] " + dict[":reply_string"])
	if dict.has_key(":object"):
	    self.ostream.write("[Output]\n" + dict[":object"].PPrint())
	self.conn.stdin_poll.Activate()
	return dict

def main():
    s = InputServer(sys.argv, "ui",
		    default_port = 15010,
		    env_class = InputEnvironment)
    s.AddDispatchFunction("reinitialize", InputEnvironment.Welcome)
    s.AddDispatchFunction("enable_input", InputEnvironment.Present)
    s.RunServer()

main()
