/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/**
 * $Id: JSAPIRecognizer.java,v 1.1.1.1 2002/01/03 18:41:39 sam Exp $
 */

package org.mitre.csms.recognition;


import galaxy.lang.*;
import galaxy.server.*;
import galaxy.server.ui.ServerUI;
import javax.speech.*;
import javax.speech.recognition.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.EventObject;
import java.beans.PropertyVetoException;
import java.net.*;
import java.io.*;

/**
 * A Communicator JSAPI compliant recognition "server". This server was
 * implemented using IBM's ViaVoice implementation of JSAPI, and
 * implements the Communicator Speech Recognitino Message Set. 
 */


public class JSAPIRecognizer extends galaxy.server.Server 
    implements CommunicatorRecognizerInterface {

    MyServerListener myl = new MyServerListener();
    static Recognizer recognizer = null;
    static RuleGrammar ruleGrammar = null;
    static DictationGrammar dictationGrammar = null;

    protected boolean notifyStatus = false;
    protected boolean attributeChangedStatus = false;    
    protected boolean interferenceStatus = false;
    protected boolean pauseAfterRecognitionStatus = false;
    protected boolean phraseStatus = false;
    protected boolean startEndSpeechStatus = false;
    protected boolean volumeLimitsStatus = false;

    protected boolean printRecognitionPropertiesDebug = false;  // debug flag
    
    public JSAPIRecognizer(MainServer mainServer, Socket socket) 
	throws IOException {
	super(mainServer, socket);
	addServerListener( myl );
    }
    // *****************************************************************
    //                  Recognition Message Set  (Receieve from Hub)
    // *****************************************************************

    public GFrame serverOpConfigure(GFrame frame) {
	if( frame.containsProperty( AUDIO_SAMPLES_KEY )) 
	    logWarning(AUDIO_SAMPLES_KEY + " not supported by JSAPI 1.0");

	if( frame.containsProperty( BACKGROUND_NOISE_LEVEL_KEY ) ) 
	    logWarning(BACKGROUND_NOISE_LEVEL_KEY + " not supported by JSAPI 1.0");
	if( frame.containsProperty( CURRENT_BYTE_KEY ))
	    logWarning(CURRENT_BYTE_KEY + " not supported by JSAPI 1.0");

	if( frame.containsProperty( INTERFERENCE_KEY )) {
	    interferenceStatus = getBoolean( frame, INTERFERENCE_KEY );
	    log(INTERFERENCE_KEY + " has been set to " + interferenceStatus);
	}

	if( frame.containsProperty( NOTIFY_ATTRIBUTE_CHANGES_KEY )) {
	    attributeChangedStatus = getBoolean( frame, NOTIFY_ATTRIBUTE_CHANGES_KEY );
	    log(NOTIFY_ATTRIBUTE_CHANGES_KEY + " has been set to " + attributeChangedStatus);
	}

	if (frame.containsProperty(NOTIFY_STATUS_KEY)) {
	    notifyStatus = getBoolean(frame, NOTIFY_STATUS_KEY );
	    log(NOTIFY_STATUS_KEY + " has been set to " + notifyStatus);
	}
	if( frame.containsProperty( PHRASE_KEY )) {
	    phraseStatus = getBoolean( frame, PHRASE_KEY );
	    log(PHRASE_KEY + " has been set to " + phraseStatus);
	}

	if( frame.containsProperty( START_END_SPEECH_KEY )) {
	    startEndSpeechStatus = getBoolean( frame, START_END_SPEECH_KEY );
	    log(START_END_SPEECH_KEY + " has been set to " + startEndSpeechStatus);
	}

	if( frame.containsProperty( VOLUME_LIMITS_KEY )) {
	    volumeLimitsStatus = getBoolean( frame, VOLUME_LIMITS_KEY );
	    log(VOLUME_LIMITS_KEY + " has been set to " + volumeLimitsStatus);
	}
	return( frame );
    }

    public GFrame serverOpForcedRecognition(GFrame frame) {
	logWarning("Forced Recognition not supported by JSAPI 1.0");
	return frame;
    }

    public GFrame serverOpGrammarControl(GFrame frame) {
	String control_type = "";
	if( frame.containsProperty( CONTROL_TYPE_KEY )) 	{
	    control_type = frame.getString( CONTROL_TYPE_KEY );
	}
	if( control_type.length() > 0 ) {
	    GVector words = null;
	    if( frame.containsProperty( PHONES_KEY )) {
		logWarning( "JSAPI 1.0 does not support adding phones");
	    }
	    if( frame.containsProperty( WORDS_KEY ))
		words = frame.getList( WORDS_KEY );
	    if ( words.size() > 0 ) {
		try {
		    boolean ruleGrammarStat = ruleGrammar.isEnabled();
		    boolean dictationGrammarStat = dictationGrammar.isEnabled();
		    ruleGrammar.setEnabled( false );
		    dictationGrammar.setEnabled( true );
		    recognizer.commitChanges();
		    for( int i=0; i<words.size(); i++ ) {
			String word  = words.elementAt( i ).toString();
			if( control_type.equalsIgnoreCase( "add" ) ) {
			    dictationGrammar.addWord( word );
			    log( "Added Word: "+word);
			}
			else if( control_type.equalsIgnoreCase( "delete" ) ) {
			    dictationGrammar.removeWord( word );
			    log( "Removed Word: "+word);
			}
		    } // for each i
		    ruleGrammar.setEnabled( ruleGrammarStat );
		    dictationGrammar.setEnabled( dictationGrammarStat );
		    recognizer.commitChanges();
		} catch ( GrammarException e ) {
		    logError("Could not commit grammar changes");
		}
	    }
	} // have new word and phones to add/delete
	return( frame );
    }

    public GFrame serverOpPause(GFrame frame) {
	new Thread( new Runnable() {
		public void run() {
		    try {
			recognizer.pause();
		    } catch (Exception ex) {
			logError("Pause error: " + ex.toString());
		    }
		}}).start();
	return( frame );
    }


    public void serverOpReinitialize(GFrame frame) {
	String grammar_file = "";
	String grammar_type = "";
	boolean commitRecognizer = false;
	RecognizerProperties recProperties = recognizer.getRecognizerProperties();
	if( frame.containsProperty( AUDIO_SOURCE_TYPE_KEY ))
	    logWarning( AUDIO_SOURCE_TYPE_KEY  + "feature not yet supported by JSAPI");
	if( frame.containsProperty( AUDIO_SOURCE_PARAM_KEY )) 
	    logWarning( AUDIO_SOURCE_PARAM_KEY  + "feature not yet supported by JSAPI");
	if( frame.containsProperty( AUTOMATIC_GAIN_KEY )) 
	    logWarning( AUTOMATIC_GAIN_KEY + " feature not yet supported by JSAPI");
	if( frame.containsProperty( ENERGY_FLOOR_KEY ))
	    logWarning( ENERGY_FLOOR_KEY + " feature not yet supported by JSAPI");
	if( frame.containsProperty( GRAMMAR_FILE_KEY ))
	    grammar_file = frame.getString( GRAMMAR_FILE_KEY );

	if( frame.containsProperty( GRAMMAR_TYPE_KEY ))
	    grammar_type = frame.getString( GRAMMAR_TYPE_KEY );

	if( frame.containsProperty( GRAMMAR_PARAMS_KEY )) {
	    logWarning( GRAMMAR_PARAMS_KEY  + "feature not supported.");
	}	
	if( grammar_type.length() > 0 ) {
	    if( (grammar_file.length() > 0) && 
		(grammar_type.equalsIgnoreCase( "rule" )) ) {
		try {
		    ruleGrammar.removeResultListener( ruleListener );
		    Reader reader = new FileReader( grammar_file );
		    ruleGrammar = recognizer.loadJSGF(reader);
		    ruleGrammar.addResultListener(ruleListener);
		    dictationGrammar.setEnabled( false );
		    ruleGrammar.setEnabled( true );
		    commitRecognizer = true;
		    log(GRAMMAR_TYPE_KEY + " has been set to " + grammar_type);
		    log(GRAMMAR_FILE_KEY + " has been set to " + grammar_file);
		} catch ( GrammarException e ) {
		    logAndSendError( "grammar exception loading rule grammar");
		} catch ( FileNotFoundException e ) {
		    logAndSendError( grammar_file + " not found");
		} catch ( IOException e ) {
		    logAndSendError( "error (io) processing grammar "+ grammar_file);
		}
	    } else if( grammar_type.equalsIgnoreCase( "dictation" )) {
		dictationGrammar.setEnabled( true );
		ruleGrammar.setEnabled( false );
		commitRecognizer = true;
		log(GRAMMAR_TYPE_KEY + " has been set to " + grammar_type);
	    } else 
		logWarning(GRAMMAR_TYPE_KEY + " not supported.");
	}
	if( frame.containsProperty( LANGUAGE_MODEL_KEY ))
	    logWarning( LANGUAGE_MODEL_KEY  + "feature not supported");
	if( frame.containsProperty( SUB_LANGUAGE_MODEL_KEY ))
	    logWarning( SUB_LANGUAGE_MODEL_KEY  + "feature not not supported");
	if( frame.containsProperty( N_BEST_KEY )) 	{
	    int numAlts = getInt( frame, N_BEST_KEY );
	    try {
		recProperties.setNumResultAlternatives( numAlts );
		commitRecognizer = true;
		log( N_BEST_KEY + " has been set to " + numAlts );
	    } catch (PropertyVetoException e ) {
		logAndSendError( "error setting property "+N_BEST_KEY );
	    }
	}

	if( frame.containsProperty( NOISE_SENSITIVITY_KEY )) 	{
	    float noiseSens = getFloat( frame, NOISE_SENSITIVITY_KEY );
	    try {
		recProperties.setSensitivity( noiseSens );
		commitRecognizer = true;
		log( NOISE_SENSITIVITY_KEY + " has been set to " + noiseSens );
	    } catch (PropertyVetoException e ) {
		logAndSendError( "error setting property "+NOISE_SENSITIVITY_KEY );
	    }
	}
	if( frame.containsProperty( PAUSE_AFTER_RECOGNITION_KEY ))
	    logWarning( PAUSE_AFTER_RECOGNITION_KEY + "feature not supported.");

	if( frame.containsProperty( REAL_TIME_SETTING_KEY )) 	{
	    float realTime = getFloat( frame, REAL_TIME_SETTING_KEY );
	    try {
		recProperties.setSpeedVsAccuracy( realTime );
		commitRecognizer = true;
		log( REAL_TIME_SETTING_KEY + "has been set to "+realTime );
	    } catch (PropertyVetoException e ) {
		logAndSendError( "error setting property "+REAL_TIME_SETTING_KEY );
	    }
	}
	if( frame.containsProperty( THRESHOLD_KEY )) 	{
	    float threshold = getFloat( frame, THRESHOLD_KEY );
	    try {
		recProperties.setConfidenceLevel( threshold );
		commitRecognizer = true;
		log( THRESHOLD_KEY + "has been set to "+threshold );
	    } catch (PropertyVetoException e ) {
		logAndSendError( "error setting property "+THRESHOLD_KEY);
	    }
	}
	if( frame.containsProperty( TIME_OUT_KEY )) 	{
	    float timeOut = getFloat( frame, TIME_OUT_KEY );
	    try {
		recProperties.setCompleteTimeout( timeOut );
		commitRecognizer = true;
		log( TIME_OUT_KEY + "has been set to "+timeOut );
	    } catch (PropertyVetoException e ) {
		logAndSendError( "error setting property "+TIME_OUT_KEY );
	    }
	}
	if( commitRecognizer ) {     // commit the recognizer
	    try {
		recognizer.commitChanges();
		printRecognizerProperties();
	    } catch ( GrammarException e ) {
		logAndSendError( "Reinitialize: error committing recognizer");
	    }
	}
    } // reinitialize

    public GFrame serverOpResume(GFrame frame) {
	new Thread( new Runnable() {
		public void run() {
		    try {
			recognizer.resume();
		    } catch (Exception ex) {
			logError("Resume error: " + ex.toString());
		    }
		}}).start();
	return( frame );
    }

    // This is the listener for server to see for example when it halts to 
    // be able to do some cleanup.  Mainly I want to know when client 
    // connections stop.


    class MyServerListener implements ServerListener {
	public void serverStarted(  Server s ) {
	    //	    System.out.println("Server started<<<<<-----");
	    try {	// setup the recognizer and get available recognizers
		EngineList list = Central.availableRecognizers(null);
		if( list.isEmpty() )
		    System.out.println( "No recognizers available" );
		if( recognizer != null )
		    recognizer.deallocate();
		recognizer = Central.createRecognizer(null);
		if( recognizer != null ) recognizer.allocate();
		recognizer.getAudioManager().addAudioListener(audioListener);
		recognizer.addEngineListener(engineListener);

		// create dictation grammar
		dictationGrammar = recognizer.getDictationGrammar(null);
		dictationGrammar.addResultListener(dictationListener);
		dictationGrammar.setEnabled(false);

		// create a rule grammar, activate it
		ruleGrammar = recognizer.newRuleGrammar( "tmp" );
		ruleGrammar.addResultListener(ruleListener);
		ruleGrammar.setEnabled(false);
		// commit new grammars, start recognizer
		recognizer.commitChanges();
		recognizer.requestFocus();
		recognizer.resume();
	    } catch (Exception e) {
		e.printStackTrace();
		System.exit(-1);
	    }
	}
				// invoked because the hub has lost connection
				// needs to take down the recognizer
	public void serverStopped( Server s ) {
	    // System.out.println( "Server stopped<<<<<<------");
	    try {
		if( recognizer != null ) {
		    recognizer.removeEngineListener( engineListener);
		    recognizer.getAudioManager().removeAudioListener(audioListener);
		    dictationGrammar.removeResultListener(dictationListener);
		    ruleGrammar.removeResultListener(ruleListener);
		    recognizer.deallocate();
		}
		recognizer = null;
	    } catch (EngineException e) {
		e.printStackTrace();
	    }
	}

	public void serverMessage( Server s, String msg ) {
	    // System.out.println( "Server Message: " + msg);	  
	    //  sendRecognitionEventMsg( "Server Message: " + msg );
	}

	public void serverWarningMessage( Server s, String msg ) {
	    // System.out.println( "Server Warning Message: " + msg);	
	    // sendRecognitionEventMsg( "Server Warning Message: " + msg );  
	}
       
	public void serverErrorMessage( Server s, String msg ) {
	    // System.out.println( "Server error message: " + msg);	  
	    // sendRecognitionEventMsg( "Server Error Message: " + msg );  
	}

	public void serverErrorMessage( Server s, String msg, Exception e ) {
	    //System.out.println( "Server error message with exception: " + msg);
	    // sendRecognitionEventMsg( "Server Error Exception Message: " + msg );  
	}

	public void serverFatalErrorMessage( Server s, String msg ) {
	    //  System.out.println( "Server Fatal error message: " + msg);	  
	}

	public void serverFatalErrorMessage( Server s, String msg, Exception e ) {
	    // System.out.println( "Server Fatal error message with exception: " + msg);
	}
	
    }; // ServerListener

    //*********************************************************************
    //	Listeners for Recognizer
    //*********************************************************************

    //
    // This is the listener for rule grammar results.
    // 
    ResultListener ruleListener = new ResultAdapter() {
	    public void resultAccepted(ResultEvent e) {
		printRecognizerProperties();
		try {
		    FinalRuleResult result = (FinalRuleResult) e.getSource();
		    String tags[] = result.getTags();
		    sendResultsMsg( tags ); // Make it here, send results msg
		} catch (Exception ex) {
		    ex.printStackTrace();
		}
	    } // results accepted

	    public void resultRejected(ResultEvent e) {
		sendInterferenceMsg( "<unknown>" );
	    } // results rejected

	    public void resultUpdated( ResultEvent e) {
		// System.out.println("Result updated");
	    }

	};

    //
    // This is the listener for dictation results.  
    // 
    ResultListener dictationListener = new ResultAdapter() {

	int n = 0; // number of tokens seen so far

	public void resultUpdated(ResultEvent e) {
	    String buffer = "";
	    Long start_time, end_time;
	    Result result = (Result) e.getSource();
	    String  msg = getResultState( result );
	    // logWarning(" result Updated "+msg);
	    for (int i=n; i<result.numTokens(); i++) {
		// System.out.println(result.getBestToken(i).getSpokenText());
		buffer = buffer + " " + result.getBestToken(i).getSpokenText();
		start_time = new Long(result.getBestToken(i).getStartTime());
		end_time = new  Long(result.getBestToken(i).getEndTime());
		sendPhraseMsg( result.getBestToken(i).getSpokenText(), start_time.toString(), end_time.toString() );
	    }
            n = result.numTokens();
	    start_time = new Long(result.getBestToken(0).getStartTime());
	    end_time = new  Long(result.getBestToken(result.numTokens()-1).getEndTime());
	    sendPhraseMsg( buffer, start_time.toString() ,end_time.toString());
	    recognizer.forceFinalize(true); // why ???
        }

        public void resultAccepted(ResultEvent e) {
	    n = 0;
	    FinalDictationResult result = (FinalDictationResult) e.getSource();
	    ResultToken firstToken = result.getBestToken( 0 );
	    ResultToken lastToken  = result.getBestToken( result.numTokens()-1 );
	    int numAlts=recognizer.getRecognizerProperties().getNumResultAlternatives();
	    ResultToken[][] altTokens=result.getAlternativeTokens( firstToken, lastToken, numAlts);
	    String tags[]  = new String[ altTokens.length ];
	    for( int i=0; i<altTokens.length; i++ ) {
		String buffer = "";
		for( int j=0; j<altTokens[i].length; j++ ) {
		    if( buffer.length() > 0 )
			buffer = buffer + " ";
		    buffer = buffer + altTokens[i][j].getSpokenText();
		}
		tags[i] = buffer;
	    }
	    sendResultsMsg( tags );
	}
    };


    //
    // Audio listener prints out audio levels to help diagnose problems.
    //
    RecognizerAudioListener audioListener =new RecognizerAudioAdapter(){
        public void audioLevel(RecognizerAudioEvent e) {
	    sendVolumeLimitsMsg( (int)e.getAudioLevel() );
	}

        public void speechStarted(RecognizerAudioEvent e) {
	    sendSpeechStartedMsg();
	}	

        public void speechStopped(RecognizerAudioEvent e) {
	    sendSpeechStoppedMsg();
	    printRecognizerProperties();
	}	
    };


    //
    // EngineListener reports engine errors, such as audio busy.
    //
    EngineListener engineListener = new EngineAdapter() {
	    public void engineError(EngineErrorEvent e) {
		logError("Engine error: " + e.getEngineError().getMessage());
		sendRecognitionEventMsg( e.getEngineError().getMessage());
	    }
	    public void enginePaused(EngineErrorEvent e) {
		log("Engine paused: " + e.getEngineError().getMessage());
		sendRecognitionEventMsg( e.getEngineError().getMessage());
	    }
	    public void engineResumed(EngineErrorEvent e) {
		log("Engine resumed: " + e.getEngineError().getMessage());
		sendRecognitionEventMsg( e.getEngineError().getMessage());
	    }
	};

				// Send messages
    protected void sendAttributeChangedMsg( String name, String newValue, String oldValue ) {
	if( attributeChangedStatus ) {
	    GFrame fr = new Clause( ATTRIBUTE_CHANGED_MSG );
	    fr.setProperty( ATTRIBUTE_NAME_KEY, name );	    
	    fr.setProperty( ATTRIBUTE_VALUE_NEW_KEY, newValue );
	    fr.setProperty( ATTRIBUTE_VALUE_OLD_KEY, oldValue );
	    writeFrame( fr );
	}
    }

    protected void sendInterferenceMsg( String interference ) {
	if( interferenceStatus ) {
	    GFrame fr = new Clause( INTERFERENCE_MSG );
	    if( (interference!=null) && (interference.length() > 0 ))
		fr.setProperty( INTERFERENCE_KEY, interference );
	    writeFrame( fr );
	}
    }

    protected void sendPhraseMsg( String phrase, String start_time, String end_time ) {
	if( phraseStatus ) {
	    GFrame fr = new Clause( PHRASE_MSG );
	    fr.setProperty( PHRASE_KEY, phrase );
	    fr.setProperty( START_TIME_KEY, start_time );
	    fr.setProperty( END_TIME_KEY, end_time );
	    writeFrame( fr );
	}
    }

    protected void sendResultsMsg( String[] tags ) {

	if( (tags != null ) && (tags.length > 0 )) {
	    GFrame fr = new Clause( RESULTS_MSG );
	    GVector vec   = new GVector( tags.length);
	    for( int i=0; i<tags.length; i++ )
		vec.addElement( (String)tags[i] );
	    fr.setProperty(RESULTS_KEY, vec ); 
	    writeFrame( fr );
	    //logWarning( "Scores not supported by JSAPI 1.0" );
	}
    }

    protected void sendSpeechStartedMsg() {
	if( startEndSpeechStatus ) {
	    GFrame fr = new Clause( SPEECH_STARTED_MSG );
	    writeFrame( fr );
	}
    }


    protected void sendSpeechStoppedMsg() {
	if( startEndSpeechStatus ) {
	    GFrame fr = new Clause( SPEECH_STOPPED_MSG );
	    writeFrame( fr );
	}
    }

    protected void sendRecognitionEventMsg(String msg) {
	if ( notifyStatus ) {
	    Clause fr= new Clause( RECOGNITION_EVENT_MSG );
	    fr.setProperty(EVENT_NAME_KEY, msg);
	    writeFrame(fr);
	}
    }

    protected void sendVolumeLimitsMsg( int volume )  {
	if( volumeLimitsStatus ) {
	    GFrame fr = new Clause( VOLUME_LIMITS_MSG );
	    fr.setProperty( VOLUME_KEY, volume );
	    writeFrame( fr );
	}
    }
				// ----------------
				// private routines
				// ----------------
    private SpeakerProfile getSpeakerProfile( String name ) {
	SpeakerManager sm = recognizer.getSpeakerManager();
	if( sm == null ) {
	    logWarning( "speaker manager is null" );
	    System.out.println(" speaker manager is null");
	}
	else {
	    SpeakerProfile sp[] = sm.listKnownSpeakers();
	    if( sp != null ) {
		for( int i=0; i<sp.length; i++ ) {
		    if( sp[i].getName().equalsIgnoreCase( name ) )
			return( sp[i] );
		}
	    }
	}
	logWarning( "profile of "+name+" not found"); 
	return( (SpeakerProfile)null );
    }

    private boolean setSpeakerProfile( String name ) {
				// not working need to fix
	SpeakerProfile sp = getSpeakerProfile( name );
	if( sp == null ) {
	    logAndSendError( "Cannot set profile for "+name );
	    return false;
	}
	else {
	    RecognizerModeDesc requireSpeaker = new RecognizerModeDesc();
	    // SpeakerProfile sp = new SpeakerProfile();
	    sp.setName( name );
	    requireSpeaker.addSpeakerProfile( sp );
		    // requireSpeaker.addSpeaker( name );
		    // see if speaker exists?
	    // if( list.anyMatch( requireSpeaker ) ) {
	    //    System.out.println("name found!!");
	    //    list.requireMatch( requireSpeaker );
	    //     } else 
	    // 	System.out.println("no name match");
	    return( true );
	}
    }
       
   private void printSpeakerProfile() {
       SpeakerManager sm = recognizer.getSpeakerManager();
       if( sm != null ) {
	   SpeakerProfile sp[] = sm.listKnownSpeakers();
	   if( sp != null ) {
	       for( int i=0; i<sp.length; i++ ) {
		   System.out.println( i + ": ( " +sp[i].getId()+ " ) " 
				       + sp[i].getName() );
	       }
	   }
       } else System.out.println("Speaker manager is null");
       
   }

    private void printRecognizerProperties() {
	if( printRecognitionPropertiesDebug ) {
	    System.out.print("      RuleGrammar: " + ruleGrammar.isEnabled() + " "+ruleGrammar.isActive());
	    System.out.print(" DictationGrammar: " + dictationGrammar.isEnabled()+ " "+dictationGrammar.isActive());
	    RecognizerProperties prop = recognizer.getRecognizerProperties();
	    System.out.print("    CompleteTimeout: " + prop.getCompleteTimeout());
	    System.out.print("    ConfidenceLevel: " + prop.getConfidenceLevel());
	    System.out.print("  IncompleteTimeout: " + prop.getIncompleteTimeout());
	    System.out.print("         NumResults: " + prop.getNumResultAlternatives());
	    System.out.print("        Sensitivity: " + prop.getSensitivity());
	    System.out.print("    SpeedVsAccuracy: " + prop.getSpeedVsAccuracy());
	    System.out.print("ResultAudioProvided: " + prop.isResultAudioProvided());
	    System.out.print("   TrainingProvided: " + prop.isTrainingProvided());
	    printRecognizerRuleGrammar();
	    System.out.println();
	}
    };

    private void printRecognizerRuleGrammar() {
	RuleGrammar RG[] = recognizer.listRuleGrammars();
	if( RG != null ) {
	    for( int i=0; i<RG.length; i++ ) {
		System.out.print( "RuleGrammar: "+ i + ": " + RG[i].toString()+ " "+RG[i].isEnabled() );
		System.out.print("            Name: " + RG[i].getName());
		System.out.print("          Active: " + RG[i].isActive());
	    }
	} else {
	    System.out.print("RuleGrammar: <null>");
	}
	if( ruleGrammar.isEnabled()) {
	    System.out.print("    Name: " +ruleGrammar.getName());
	    System.out.print("  Active: "+ruleGrammar.isActive());
	}
    }
	    
    private String getResultState( Result result ) {
	String msg = "UNKNOWN";
	if( result.getResultState() == Result.UNFINALIZED )
	    msg = "UNFINALIZED";
	else if( result.getResultState() == Result.ACCEPTED )
	    msg = "ACCEPTED";
	else if( result.getResultState() == Result.REJECTED )
	    msg = "REJECTED";
	return( msg );
    }
    
    private float getFloat( GFrame frame, String key ) {
	return( frame.getFloat(key).floatValue() );
    }

    private int getInt( GFrame frame, String key ) {
	return( frame.getInt(key).intValue() );
    }

    private boolean getBoolean( GFrame frame, String key ) {
	int ivalue = getInt( frame, key );
	if( ivalue > 0)
	    return( true );
	else
	    return( false );
    }

    public static void main(String[] args) {	
	ServerUI ui = new ServerUI(args) {
		protected void init() {
		    serverClassName = "org.mitre.csms.recognition.JSAPIRecognizer";
		    port = 15560;
		}};
    }

}

