/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

// $Id: JSAPISynthesizer.java,v 1.1.1.1 2002/01/03 18:41:40 sam Exp $

package org.mitre.csms.synthesis;

import galaxy.lang.*;
import galaxy.server.*;
import galaxy.server.ui.ServerUI;
import javax.speech.synthesis.*;
import javax.speech.*;
import java.io.*;
import java.net.*;

/**
 * A Communicator JSAPI compliant synthesis "server". This server was
 * implemented using IBM's ViaVoice implementation of JSAPI, and
 * implements the Communicator Speech Synthesis Message Set. 
 */

public class JSAPISynthesizer extends galaxy.server.Server 
    implements CommunicatorSynthesizerInterface {

    MyServerListener myl = new MyServerListener();
    static Synthesizer synthesizer = null;
  
  
    protected boolean notifyStatus = false;
    protected boolean attributeChangeStatus = false;
    protected boolean wordPositionStatus = false;
    protected boolean markerStatus = false;

    public JSAPISynthesizer(MainServer mainServer, Socket socket)
	throws IOException {
	super(mainServer, socket);
	addServerListener( myl );
    }

    public void serverOpReinitialize(GFrame frame) {
	String stat = engineState();
	if( stat.length() > 0 )
	    System.out.print("Reinitialize: Engine State: "+stat);
	else 
	    System.out.println("Reinitialize  Engine State: unknown: ");


	SynthesizerProperties synthProps = synthesizer.getSynthesizerProperties();
	if (frame.containsProperty(AUDIO_DESTINATION_TYPE_KEY)) {
	    logWarning(AUDIO_DESTINATION_TYPE_KEY + " feature not yet supported by JSAPI");
	}
	if (frame.containsProperty(AUDIO_DESTINATION_PARAM_KEY)) {
	    logWarning(AUDIO_DESTINATION_PARAM_KEY +" feature not yet supported by JSAPI");
	}
	if (frame.containsProperty(AVERAGE_SPEAKING_SPEED_KEY)) {
	    logWarning(AVERAGE_SPEAKING_SPEED_KEY+" cannot be set with ViaVoice");
	    /*-------------
	    try {
		synthProps.setSpeakingRate(frame.getFloat(AVERAGE_SPEAKING_SPEED_KEY).floatValue());
	    } catch (java.beans.PropertyVetoException pve) {
		logError(AVERAGE_SPEAKING_SPEED_KEY,pve);
	    }
	    ---------------- */
	}

	if (frame.containsProperty(PHONEME_TRANSLATION_FILE_KEY)) {
	    logWarning(PHONEME_TRANSLATION_FILE_KEY + " not applicable to JSAPI");
	}
	
	if (frame.containsProperty(PITCH_KEY)) {
	    logWarning(PITCH_KEY+" cannot be set with ViaVoice");
	    /* -------------
	    try {
		synthProps.setPitch(frame.getFloat(PITCH_KEY).floatValue());
	    } catch (java.beans.PropertyVetoException pve) {
		logError(PITCH_KEY,pve);
	    }
	    ----------------- */
	}

	if (frame.containsProperty(REAL_TIME_SETTING_KEY)) {
	    logWarning(REAL_TIME_SETTING_KEY + " : not implemented in JSAPI 1.0");
	}

	// Float that ranges from 0.0 to 1.0 in IBM's ViaVoice implementation
	if (frame.containsProperty(VOLUME_KEY)) {
	    logWarning(VOLUME_KEY+" cannot be set with ViaVoice");
	    /* ---------------
	    try {
		synthProps.setVolume(frame.getFloat(VOLUME_KEY).floatValue());
	    } catch (java.beans.PropertyVetoException pve) {
		logError(VOLUME_KEY,pve);
	    }
	    ------------------ */
	}
    }

    public GFrame serverOpGetVoice(GFrame frame) {
	logWarning("GetVoice not implemented yet");
	return frame;
    }


    public GFrame serverOpSetVoice(GFrame frame) {
	logWarning("SetVoice not implemented yet");
	return frame;
    }

    
    public GFrame serverOpConfigure(GFrame frame) {

	if (frame.containsProperty(MARKER_KEY)) {
	    markerStatus = getBoolean(MARKER_KEY, frame);
	    log(MARKER_KEY + " has been set to " + markerStatus);
	}

	if (frame.containsProperty(CURRENT_BYTE_KEY)) {
	    logWarning(CURRENT_BYTE_KEY+" feature not yet supported by JSAPI");
	}

	if (frame.containsProperty(NOTIFY_ATTRIBUTE_CHANGES_KEY)) {
	    attributeChangeStatus = getBoolean(NOTIFY_ATTRIBUTE_CHANGES_KEY, frame);
	    log(NOTIFY_ATTRIBUTE_CHANGES_KEY + " has been set to " + attributeChangeStatus);
	}

	if (frame.containsProperty(NOTIFY_STATUS_KEY)) {
	    notifyStatus = getBoolean(NOTIFY_STATUS_KEY, frame);
	    log(NOTIFY_STATUS_KEY + " has been set to " + notifyStatus);
	}

	if (frame.containsProperty(NOTIFY_WORD_POSITION_KEY)) {
	    wordPositionStatus = getBoolean(NOTIFY_WORD_POSITION_KEY, frame);
	    log(NOTIFY_WORD_POSITION_KEY + " has been set to " + wordPositionStatus);
	}

	return frame;
    }
    
    public GFrame serverOpPause(GFrame frame) {
	new Thread( new Runnable() {
		public void run() {
		    try {
			synthesizer.pause();
		    } catch (Exception ex) {
			// logAndSendError(ex.toString());
			logError("Pause error: " + ex.toString());
		    }
		}}).start();
	return frame;
    }

    public GFrame serverOpPronunciation(GFrame frame) {
	logWarning("Pronunciation not supported by JSAPI 1.0, use JSML instead");
	return frame;
    }

    public GFrame serverOpGetWordPronunciation(GFrame frame) throws EngineStateError {
				// be sure that queue empty first!
	try {
	    if( !synthesizer.testEngineState( Synthesizer.QUEUE_EMPTY ) )
		synthesizer.waitEngineState( Synthesizer.QUEUE_EMPTY );
	} catch (InterruptedException ex) {
	    logError("error testing engine state: " + ex.toString());
	}

	String pronun="";
	if ( frame.containsProperty(WORD_KEY) ) {
	    pronun = synthesizer.phoneme(frame.getString(WORD_KEY));
	}
	frame.setProperty(PHONE_SEQUENCE_KEY,pronun);
	return frame;
    }

    public GFrame serverOpRemoveWord(GFrame frame) {
	logWarning("Remove not supported by JSAPI 1.0, use JSML instead");
	return frame;
    }

    public GFrame serverOpResume(GFrame frame) {
	new Thread( new Runnable() {
		public void run() {
		    try {
			synthesizer.resume();
		    } catch (Exception ex) {
			logError("Resume error: " + ex.toString());
		    }
		}}).start();
	return frame;
    }

    public GFrame serverOpSpeak(GFrame frame) throws javax.speech.synthesis.JSMLException {
	if (frame.containsProperty(TEXT_KEY)) {
	    if (synthesizer!=null) {
		try {
		    synthesizer.speak(frame.getString(TEXT_KEY), null);
		} catch (EngineStateError ese) {
		    ese.printStackTrace();
		}
	    }
	} else {
	    logWarning("speak invoked with no text!!");
	}
	return frame;
    }

    protected void sendSynthesisEventMsg(String msg) {
	if ((notifyStatus) && (msg != null)) {
	    GFrame fr= new Clause(SYNTHESIS_EVENT_MSG);
	    fr.setProperty(EVENT_NAME_KEY, msg);
	    writeFrame(fr);
	}
    }

    protected void sendMarkerReachedMsg(String msg) {
	if (notifyStatus && markerStatus && (msg != null)) {
	    GFrame fr = new Clause(MARKER_REACHED_MSG);
	    fr.setProperty(MARKER_KEY, msg);
	    writeFrame(fr);
	}
    }

    protected void sendWordPosition(int c) {
	if (notifyStatus && wordPositionStatus) {
	    GFrame fr = new Clause(WORD_POSITION_MSG);
	    fr.setProperty(WORD_POSITION_KEY, c);
	    writeFrame(fr);
	}
    }

    private boolean getBoolean(String key, GFrame frame) {
	int status = frame.getInt(key).intValue();
	if (status > 0)
	    return true;
	else
	    return false;
    }

    private String engineState() {
	String stat = "";
	long engstate = synthesizer.getEngineState();
	if ((engstate  & Engine.ALLOCATED) > 0) 
	    stat = stat + "allocated: ";
	if ((engstate  & Engine.DEALLOCATED) > 0) 
	    stat = stat + "de-allocated: ";
	if ((engstate  &  Engine.RESUMED) > 0)
	    stat = stat + "resumed: ";
	if ((engstate  & Engine.PAUSED) > 0)
	    stat = stat + "paused: ";
	if ((engstate  & Synthesizer.QUEUE_NOT_EMPTY ) > 0)
	    stat = stat + "queue not empty: ";
	if ((engstate  & Synthesizer.QUEUE_EMPTY ) > 0)
	    stat = stat + "queue empty: ";
	return stat;
    }

    // This is the listener for server to see for example when it halts to 
    // be able to do some cleanup.  Mainly I want to know when client 
    // connections stop.

    class MyServerListener implements ServerListener {
	public void serverStarted(  Server s ) {
	    // System.out.println("Server started<<<<<-----");
	    try {
		// create a synthesizer, speak a greeting
		synthesizer = Central.createSynthesizer(null);
		if (synthesizer!=null) {
		    synthesizer.allocate();
		    synthesizer.addEngineListener(engineListener);
		    synthesizer.addSpeakableListener(speakableListener);
		    synthesizer.resume();
		}
		else {
		    System.out.println("Cannot setup synthesizer (null)");
		    System.exit(-1);
		}
	    } catch (IllegalArgumentException iae) {
		iae.printStackTrace();
		System.exit(-1);
	    } catch (EngineException ee) {
		ee.printStackTrace();
		System.exit(-1);
	    } catch (Exception ex) {
		logError("Resume error: " + ex.toString());
		System.exit(-1);
	    }
	}
				// invoked because the hub has lost connection
				// needs to take down the recognizer
	public void serverStopped( Server s ) {
	    // System.out.println( "Server stopped<<<<<<------");
	    try {
		if( synthesizer != null ) {
		    synthesizer.removeEngineListener( engineListener);
		    synthesizer.removeSpeakableListener(speakableListener);
		    synthesizer.deallocate();
		}
		synthesizer = null;
	    } catch (EngineException e) {
		e.printStackTrace();
	    }
	}

	public void serverMessage( Server s, String msg ) {
	    // System.out.println( "Server Message: " + msg);	  
	}

	public void serverWarningMessage( Server s, String msg ) {
	    // System.out.println( "Server Warning Message: " + msg);	
	}
       
	public void serverErrorMessage( Server s, String msg ) {
	    // System.out.println( "Server error message: " + msg);	  
	}

	public void serverErrorMessage( Server s, String msg, Exception e ) {
	    // System.out.println( "Server error message with exception: " + msg);
	}

	public void serverFatalErrorMessage( Server s, String msg ) {
	    //  System.out.println( "Server Fatal error message: " + msg);	  
	}

	public void serverFatalErrorMessage( Server s, String msg, Exception e ) {
	    // System.out.println( "Server Fatal error message with exception: " + msg);
	}
    }; // ServerListener
  
    /*
     * SpeakableListener reports engine status, such as which word is currently being processed.
     */
    SpeakableListener speakableListener = new SpeakableAdapter() {
	    int wordCounter = 0;
	    public void markerReached(SpeakableEvent e) {
		if (notifyStatus && markerStatus) {
		    String text = e.getText();
		    sendMarkerReachedMsg(text);
		}
	    }

	    public void  speakableStarted(SpeakableEvent e) {
		wordCounter = 0;
	    }

	    public void  wordStarted(SpeakableEvent e) {
		wordCounter++;
		sendWordPosition(wordCounter);
	    }
	};
    /**
     * EngineListener reports engine errors, such as audio busy.
     */
    EngineListener engineListener = new EngineAdapter() {
	    public void engineError(EngineErrorEvent e) {
		logError("Engine error: " + e.getEngineError().getMessage());
		sendSynthesisEventMsg(e.getEngineError().getMessage());
	    }
	    public void enginePaused(EngineErrorEvent e) {
		log("Engine paused: " + e.getEngineError().getMessage());
		sendSynthesisEventMsg(e.getEngineError().getMessage());
	    }
	    public void engineResumed(EngineErrorEvent e) {
		log("Engine resumed: " + e.getEngineError().getMessage());
		sendSynthesisEventMsg(e.getEngineError().getMessage());
	    }
	};



    public static void main(String[] args) {
	// Setup Communicator Server...
	ServerUI ui = new ServerUI(args) {
		protected void init() {
		    serverClassName = "org.mitre.csms.synthesis.JSAPISynthesizer";
		    port = 15570;
		}};
    }
}
