/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

/*****************************************************************************
  NOTICE
   
  A portion of this software was produced for the U.S. Government
  under Contract No. DAAB07-97-C-E601, and is
  subject to the Rights in Technical Data and Computer
  Software Clause 252.227-7013 (Oct 88) and Alternate II (APR 93).
  
  Copyright 1998 The MITRE Corporation
 *****************************************************************************/

/****************************************************************************
* say:  Converts text message to audio message in the hub
*
*       modified by Alan W Black (awb@cstr.ed.ac.uk)
*       based heavily on the example truetalk server which was 
*       written by: Alan Goldschen
*
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include "FestivalAPI.h"
#define SERVER_FUNCTIONS_INCLUDE "festserv_server.h"
#define USE_SERVER_DATA
#include "galaxy/server_functions.h"
#include "audioCommon.h"
#include "galaxy/galaxy_all.h"

/*****************************************************************************
**  Global Types (local to file)
******************************************************************************/

static Gal_Boolean Trace = GAL_FALSE;

#define FESTIVAL_DEFAULT_HOST "localhost"

/******************************************************************************
**  Internal Functions (Methods) Definitions
******************************************************************************/

static void quitHandler(int sig);
static int processSpeech(char *buf8, int nbytes, int sample_rate, int rec_size);
static int startProcessSpeech();
static int endProcessSpeech(int sample_rate);
static void SaveInt16(short *data, int size, Gal_Boolean done, int sample_rate);
static void SaveBinary(char *data, int size, Gal_Boolean done, int sample_rate);
/* static int BrokerBinary( int size, unsigned char *data ); */
static Gal_Frame prepare_audio_frame( int size, void *data, int sample_rate, Gal_Boolean done);

/******************************************************************************
**  Export Functions (Methods) accessiable by outside routines
******************************************************************************/

/*****************************************************************************
**  Global Vars (local to file)
******************************************************************************/

static FT_Info *FTConfig = (FT_Info *) NULL;

/******************************************************************************
**  Export Functions (Methods) accessiable by outside routines
******************************************************************************/


Gal_Frame FestivalConfig(Gal_Frame f, void *server_data)
{
  /** Trap Errors **/
  signal(SIGQUIT, quitHandler);
  signal(SIGINT, quitHandler);
  signal(SIGTERM, quitHandler);
  signal(SIGSEGV, quitHandler);
  signal(SIGBUS, quitHandler);

  /* Recognizes three flags: :linear, :playrate. :playgain
     is no longer recognized, because it's ignored. */

  /*
    #if 0
    if (!festivalCBConfig(FTConfig,
    Gal_GetString(f, ":encoding"),
    processSpeech)) {
    printf("error configuring festival\n");
    exit(-1);
    }
    #endif
  */
  return( f );
}

/* SAM 1/5/98: This isn't going to work in threads, but here
   it will work OK. I'll use a static global to store the 
   comm object, since the TT API doesn't allow any extra
   arguments to the callback. */
  
static GalSS_Environment *TTGComm;
static char *CurSessionID = (char *) NULL;

Gal_Frame speak_output(Gal_Frame f, void *server_data)
{
  char *msg;
  FT_Wave *wave;

  TTGComm = (GalSS_Environment *) server_data;

  /* This method should set: ":synth_port, :synth_host, :call_id" and
     call on program synthesis to be compatible with MIT's audio server 
  */

  if (Gal_GetString(f, GAL_SESSION_ID_FRAME_KEY)) {
    CurSessionID = Gal_GetString(f, GAL_SESSION_ID_FRAME_KEY);
  } else {
    CurSessionID = (char *) NULL;
  }

  msg = Gal_GetString(f, ":reply_string");
  if( (msg == NULL ) || (msg[0]=='\0') || (msg[0]==0x00) ) {
    printf("msg is null\n");
  } else {
    printf("msg is: <%s>\n", msg );

    wave = festivalStringToWave(FTConfig,msg);

    /* Send start signal */
    startProcessSpeech();
    /* Send data */
    processSpeech((char *)wave->samples, wave->num_samples*sizeof(short), wave->sample_rate, 0);
    /* Send end signal */
    endProcessSpeech(wave->sample_rate);
 
    delete_FT_Wave(wave);
  }

  return (Gal_Frame) f;
}

Gal_Frame reinitialize(Gal_Frame frame, void *server_data )
{
  /* Send everything to Festival for its configuration */
  FestivalConfig(frame, server_data);
  printf("return from festival config\n");
  return( frame );
}

/******************************************************************************
**  Internal Functions
******************************************************************************/

#define MIN2(a,b)  (((a) < (b)) ? (a) : (b))
#define TRANSFER_SIZE   4096

static int processPCMSpeech(short *p8b, int nSamples, int sample_rate)
{
  static short buf16b8Khz[TRANSFER_SIZE];
  /* static short buf16b16Khz[2*TRANSFER_SIZE]; */
  static short *p16b;
  /* static short initial; */
  static int numComplete, numSamples, numProcess;
  /* static char cbuf8b16Khz[2*TRANSFER_SIZE]; */
  int i;
  
  if(!p8b) {
    /* initial = 0;*/
    numComplete = 0;
    p16b = buf16b8Khz;
    fprintf(stderr, "S");
    return 1;
  }
  else if(nSamples == 0) {
    fprintf(stderr, "E\n");
    /* conv8Khzto16Khz_16b(buf16b8Khz, numComplete, &initial, buf16b16Khz);
       conv16bto8b(buf16b16Khz, 2*numComplete, cbuf8b16Khz ); */
      
    /* let's save up binary and send all at once */
    /* SAM 12/31/98: Changed this to send unmodified
       waveforms. */
    /* SaveBinary( (char *)cbuf8b16Khz, 2*numComplete, GAL_TRUE); */
    SaveInt16(buf16b8Khz, numComplete, GAL_TRUE, sample_rate);
    return 1;
  }
  numSamples = nSamples;
  while(numSamples) {
    numProcess = MIN2(numSamples, TRANSFER_SIZE - numComplete);
    for(i = 0; i < numProcess; i++)
      *p16b++ = *p8b++;
      
    numSamples -= numProcess;
    numComplete += numProcess;
      
    if(numComplete == TRANSFER_SIZE) {
      fprintf(stderr, "M");
      /* conv8Khzto16Khz_16b(buf16b8Khz, numComplete, &initial, buf16b16Khz);
	 conv16bto8b(buf16b16Khz, 2*numComplete, cbuf8b16Khz ); */
	  
      /* let's save up binary and send all at once */
      /* SAM 12/31/98: Changed this to send unmodified
	 waveforms. */
      /* SaveBinary( (char *)cbuf8b16Khz, 2*numComplete, GAL_FALSE, sample_rate); */
      SaveInt16(buf16b8Khz, numComplete, GAL_FALSE, sample_rate);
	  
      /* initial = 0; */
      numComplete = 0;
      p16b = buf16b8Khz;
    }
  }
  
  return 1;
}


static int processMulawSpeech(char *p8b_in, int nbytes, int sample_rate)
{
  /* static short buf16b8Khz[TRANSFER_SIZE], buf16b16Khz[2*TRANSFER_SIZE]; */
  /* static short *buf8b16Khz = buf16b8Khz, *p16b, *p8b; */
  static char buf8b8Khz[TRANSFER_SIZE];
  static char *p8b;
  /* static char initial; */
  static int numComplete, numSamples, numProcess;
  /* static char cbuf8b16Khz[2*TRANSFER_SIZE]; */
  int i;
  
  if(!p8b_in) {
    /* initial = 0; */
    numComplete = 0;
    p8b = buf8b8Khz;
    fprintf(stderr, "S");
    return 1;
  }
  else if(nbytes == 0) {
    fprintf(stderr, "E\n");
    /* conv8Khzto16Khz_16b(buf16b8Khz, numComplete, &initial, buf16b16Khz);
       conv16bto8b(buf16b16Khz, 2*numComplete, cbuf8b16Khz ); */
      
    /* let's save up binary and send all at once */
    /* SAM 12/31/98: Changed this to send unmodified
       waveforms. */
    /* SaveBinary( (char *)cbuf8b16Khz, 2*numComplete, GAL_TRUE, sample_rate); */
    SaveBinary(buf8b8Khz, numComplete, GAL_TRUE, sample_rate);
    return 1;
  }
  
  p8b = p8b_in;
  numSamples = nbytes;

  while(numSamples) {
    numProcess = MIN2(numSamples, TRANSFER_SIZE - numComplete);
    for(i = 0; i < numProcess; i++)
      *p8b++ = *p8b_in++;
      
    numSamples -= numProcess;
    numComplete += numProcess;
      
    if(numComplete == TRANSFER_SIZE) {
      fprintf(stderr, "M");
      /* conv8Khzto16Khz_16b(buf16b8Khz, numComplete, &initial, buf16b16Khz);
	 conv16bto8b(buf16b16Khz, 2*numComplete, cbuf8b16Khz ); */
	  
      /* let's save up binary and send all at once */
      /* SAM 12/31/98: Changed this to send unmodified
	 waveforms. */
      /* SaveBinary( (char *)cbuf8b16Khz, 2*numComplete, GAL_FALSE); */
      SaveBinary(buf8b8Khz, numComplete, GAL_FALSE, sample_rate);
	  
      /* initial = 0; */
      numComplete = 0;
      p8b = buf8b8Khz;
    }
  }
  
  return 1;
}

static int processSpeech(char *buf8, int nbytes, int sample_rate, int rec_size)
{
  (void)rec_size;    
  return processPCMSpeech((short *) buf8, nbytes/2, sample_rate);
  
  /*     #if 0 */
  /*     if (1 || (FTConfig->encoding == ENCODING_PCM) ) */
  /*     { */
  /* array of shorts */ 
  /*   return processPCMSpeech((short *) buf8, nbytes/2, sample_rate); */
  /* } else { */
  /* array of bytes */ 
  /*   return processMulawSpeech(buf8, nbytes, sample_rate); */
  /* } */
  /* #endif */

}

static int startProcessSpeech() {
  return processSpeech((char *) 0, 0, 0, 0);
}

static int endProcessSpeech(int sample_rate) {
  return processSpeech((char *) 1, 0, sample_rate, 0);
}

static void SaveBinary(char *data, int size, Gal_Boolean done, int sample_rate)
{
#define LEN 50000 
  static short bigbuffer[LEN];
  static int biglen = 0;

  int i, lim;
  
  
  if( (data == NULL )  || (size <= 0) ) {
    printf("SaveBinary: null data or bad size\n");
    return;
  }
  lim = size;

  if( biglen+size > LEN )
    lim = LEN - biglen - 5;

  for( i=0; i<lim; i++ )
    bigbuffer[biglen+i] = data[i];

  biglen = biglen + lim;

  if( (biglen > (LEN - 2000)) || (done == GAL_TRUE) ) {
    Gal_Frame f;

    f = prepare_audio_frame( biglen, (void *) bigbuffer, sample_rate, done );
    if( f == (Gal_Frame)NULL  && done ) {
      if( Trace )
	printf("SaveBinary: error setting up brokering .. no data send\n");
    } else {			/* brokering okay */
      if( Trace )
	printf("SaveBinary: brokering setup okay\n");
      fprintf(stderr, "\nSending binary of size %d to playbox\n\n", biglen);
      GalSS_EnvWriteFrame(TTGComm, f, 0);
    }
    biglen = 0;
  }
}

static void SaveInt16(short *data, int size, Gal_Boolean done, int sample_rate)
{
#define LEN 50000
  static short bigbuffer[LEN];
  static int biglen = 0;

  int i, lim;
  
  
  if( (data == NULL )  || (size <= 0) ) {
    printf("SaveBinary: null data or bad size\n");
    return;
  }
  lim = size;

  if( biglen+size > LEN )
    lim = LEN - biglen - 5;

  for( i=0; i<lim; i++ )
    bigbuffer[biglen+i] = data[i];

  biglen = biglen + lim;

  if( (biglen > (LEN - 2000)) || (done == GAL_TRUE) ) {
    Gal_Frame f;

    f = prepare_audio_frame( biglen, (void *) bigbuffer, sample_rate, done);
    if( f == (Gal_Frame)NULL && done) {
      if( Trace )
	printf("SaveBinary: error setting up brokering .. no data send\n");
    }
    else {			/* brokering okay */
      if( Trace )
	printf("SaveBinary: brokering setup okay\n");
      fprintf(stderr, "\nSending binary of size %d to playbox\n\n", biglen);
      if (f)
	GalSS_EnvWriteFrame(TTGComm, f, 0);
    }
    biglen = 0;
  }
}


static Gal_Frame prepare_audio_frame( int size, void *data, int sample_rate, Gal_Boolean done)
{
  static Gal_Frame f;
  static GalIO_BrokerStruct *b;
  char host_pid[1024];

  if( size <= 0 ) {
    printf("prepare_audio: invalid size .. no data to send\n");
    return( (Gal_Frame)NULL );
  }
  if( data == NULL ) {
    printf("prepare_audio: data is null .. \n");
    return( (Gal_Frame)NULL );
  }

  /* 
   * If we're the sending direction, we build a binary data structure. 
   * Once we do that, we transmit the data over a socket by transmitting the 
   * size of the memory buffer, and then transmitting the contents of the
   * memory buffer. 
   */

  /* Now that we have the audio, we add a binary element. */
  sprintf(host_pid, "%s:%d", GalIO_IPAddress(), (int) getpid());
  /* 
   * The following keys have been modified:
   *  synthesis_binary_host --> synth_host
   *  synthesis_binary_port --> synth_port
   * Changed the name of the program from "main" to "synthesis"
   * Added :sample_rate and :encoding_format to the frame.
   * All this to become compliant with MIT's "standard"
   * by Sasha
   *
   */

  if (!f) {
    f = Gal_MakeFrame("synthesis", GAL_CLAUSE);
    Gal_MakeFrame("synthesis", GAL_CLAUSE);
    if (CurSessionID) {
      Gal_SetProp(f, GAL_SESSION_ID_FRAME_KEY, Gal_StringObject(CurSessionID));
    }
  }
  
  if (!b) {
    b = GalIO_BrokerDataOutInit(GalSS_EnvComm(TTGComm), 0, 10);  
    if (b && (GalIO_GetBrokerListenPort(b) > 0)) {
      GalIO_BrokerPopulateFrame(b, f, ":synth_host", ":synth_port");
      Gal_SetProp(f, ":encoding_format", Gal_StringObject("linear16"));  
      Gal_SetProp(f, ":sample_rate", Gal_IntObject(sample_rate));
      GalIO_BrokerWriteString(b, AUDIO_START);
    }
  }
  GalIO_BrokerWriteInt16(b, data, size);

  if (done) {
    Gal_Frame f2;
    f2 = Gal_CopyFrame(f);
    GalIO_BrokerWriteString(b, AUDIO_END);
    GalIO_BrokerDataOutDone(b);
    b = NULL;
    f = NULL;
    return f2;
  }
  return( (Gal_Frame)NULL );
}

static void quitHandler(int sig)
{
  printf("Exiting with signal %d\n", sig);

  fflush(stderr);
  fflush(stdout);
  /*  disconnectMessage(); */
  festivalClose(FTConfig);
  
  exit(1);
}

/* Command line interpretation */
static char *oas[] = {
  "-tts_host host", "host where the festival server (festival -server) is running (default: localhost)", FESTIVAL_DEFAULT_SERVER_HOST,
  "-tts_port port", "port on which festival server is running", NULL,
  "-text_mode mode", "Text mode used by festival (default: fundamental)", FESTIVAL_DEFAULT_TEXT_MODE,
  NULL
};

/* This serveris multiple-connection safe, but not thread-safe,
   due to the global callbacks. */

void *_GalSS_init_server(GalIO_ServerStruct *s, int argc, char **argv)
{
  int i;
  
  FTConfig = (FT_Info *)calloc(1, sizeof(FT_Info));
  
  if (!GalUtil_OACheckUsage(argc, argv, oas, &i))
    exit(1);

  /**
   * Initialze FTConfig
   */
  FTConfig->server_fd = GAL_INVALID_SOCKET;
  GalUtil_OAExtract(argc, argv, oas, "-tts_host", GAL_OA_STRING, 
		    &FTConfig->server_host);
  if(!GalUtil_OAExtract(argc, argv, oas, "-tts_port", GAL_OA_INT, &FTConfig->server_port))
    FTConfig->server_port = FESTIVAL_DEFAULT_SERVER_PORT;
  GalUtil_OAExtract(argc, argv, oas, "-text_mode", GAL_OA_STRING, 
	     &FTConfig->text_mode);
  
  FTConfig = festivalOpen(FTConfig);

  if (!FTConfig) {
    fprintf(stderr,"Error: Connection to festival server failed!\n");
    fflush(stderr);
    exit(1);
  } else {
    printf("festival open\n");
  }
  return (void *) NULL;
}
