/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "truetalkAPI.h"

#define DEFAULT_TTS_HOST "astral"

static TTserver server;
static TTclient clienT;

static unsigned int host_port[2] = {0, 0};

#ifdef TRUETALK20
static void index_func( TTclient client, TTevent event, int marker, int sample_in_bytes )
{
  printf("in index function to something with marker\n");
  return;
}
#endif

int truetalkOpen(char *host)
{
   int status;

#ifdef TRUETALK20
   status = TT_Init(NULL);
   if( status == 0 ) {
        fprintf(stderr, "TT_Init failed: %s", TT_DecodeLastError());
	fflush(stderr);
	exit(-1);
   }
#else
   status = EinitTtbase(NULL);
   if (status != 0) {
        fprintf(stderr, "TT_Init failed: %s", TT_DecodeLastError());
	fflush(stderr);
	exit(-1);
   }
#endif

   else {
     printf("Truetalk API has been initialized\n");
     fflush(stdout);
   }   

   /* Added by SPC */
   /* printf("Right here\n"); */
   if (host == NULL || !strcmp(host, ""))
     host = strdup(DEFAULT_TTS_HOST);

   /* printf("Passed!"); */
   printf("Binding to server on %s with port %d\n", host, host_port[0]);
   fflush(stdout);
   if((server = TT_BindServer(host, host_port)) == NULL)
   {
      fprintf(stderr, "TT_BindServer failed: %s\n", TT_DecodeLastError());
      fflush(stderr);
      exit(1);
   }
   else {
     printf("Bound to TT server\n");
     fflush(stdout);
   }

   if((clienT = TT_Open(server)) == NULL)
   {
      fprintf(stderr, "TT_Open failed: %s\n", TT_DecodeLastError());
      fflush(stderr);
      exit(1);
   }
   else {
     printf("TT opened\n");
     fflush(stdout);
   }

   return 1;
}


#ifdef TRUETALK20
int truetalkCBConfig(TT_DATA *data, char *encoding,
		     int (*callBack) (char*, int, int ))
#else
int truetalkCBConfig(TT_DATA *data, char *encoding,
		     int (*callBack) (char*, int, int, struct TtsIndex*))
#endif
{
  data->encoding = ENCODING_MULAW;

  if (encoding) {
    if (!strcmp(encoding, "linear16")) {
      data->encoding = ENCODING_PCM;
    } else if (!strcmp(encoding, "mulaw255")) {
      data->encoding = ENCODING_MULAW;
    } else {
      fprintf(stderr, "truetalkConfig: Invalid encoding '%s', must be linear16 or mulaw255", encoding);
      fprintf(stderr, "truetalkConfig: Setting encoding to 8-bit mulaw\n");
      fflush(stderr);
      data->encoding = ENCODING_MULAW;
    }
  }
  if(TT_SetOutput(clienT, OUTPUT_CLIENT, OUTPUT_FUNC, data->encoding, 8000, (void *)callBack) == 0) {
    fprintf(stderr, "TT_SetOutput failed: %s\n", TT_DecodeLastError());
    fflush(stderr);
    truetalkClose();
    return 0;
  }

#ifdef TRUETALK20
  if(TT_SetEventCallback( clienT, TT_Index, index_func ) == 0) {
    fprintf(stderr,"TT_SetEventCallback failed: %s\n", TT_DecodeLastError());
    fflush(stderr);
    truetalkClose();
    return 0;
  }
#endif
  return 1;
}


int truetalkSay(char *text)
{
   long marker;

   if((marker = TT_SendText(clienT, text, strlen(text))) == 0)
   {
      fprintf(stderr, "TT_SendText failed: %s\n", TT_DecodeLastError());
      fflush(stderr);
      truetalkClose();
      exit(1);
   }
   
   if(TT_Sync(clienT, marker) == 0)
   {
      fprintf(stderr, "TT_Sync failed: %s\n", TT_DecodeLastError());
      fflush(stderr);
      truetalkClose();
      exit(1);
   }
   return 1;
}


int truetalkClose()
{
#ifdef TRUETALK20
   if(TT_Close(clienT, server) == 0)
#else
   if(TT_Close(clienT) == 0)
#endif
   {
      fprintf(stderr, "TT_Close failed: %s\n", TT_DecodeLastError());
      fflush(stderr);
      exit(1);
   }

#ifdef TRUETALK20
   if( TT_Restore() == 0) {
     fprintf(stderr, "TT_Restore failed: %s\n", TT_DecodeLastError());
     fflush(stderr);
     exit(1);
   } 
#endif
   return 1;
}
