# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import os, sys, string


sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy, GalaxyIO

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE",
				"servers", "src", "frame_transducer"))

from transducer_core import TransducerMapDB

OAS = [("-default_config_file <str>", "default configuration file", [type("")]),
       ("-transducer_debug", "Enable verbose transducer output", [])]
	

class TransducerServer(GalaxyIO.Server):
    def __init__(self, *args, **kw):
	apply(GalaxyIO.Server.__init__, (self,) + args, kw)
	self.default_map_db = None
	self.map_db_table = {}
	self.transducer_verbose = 0

    def CheckUsage(self, oas_list, args):
	data, out_args = GalaxyIO.Server.CheckUsage(self, OAS + oas_list, args)
	if data.has_key("-default_config_file"):
	    config_file = data["-default_config_file"][0]	    
	    del data["-default_config_file"]
	    try:
		self.default_map_db = TransducerMapDB(config_file)
	    except SyntaxError:
		print "Encountered an error loading configuration file."
		sys.exit(1)
	if data.has_key("-transducer_debug"):
	    self.transducer_verbose = 1
	    del data["-transducer_debug"]
	return data, out_args

# The :config_files are a list of files.

class TransducerEnvironment(GalaxyIO.CallEnvironment):

    def Welcome(self, dict):
	# If there's a configuration file, use it instead of
	# the default one.
	if dict.has_key(":config_files"):
	    l = dict[":config_files"]
	    if type(l) is not type([]):
		print ":config_files is not a list of files."
	    else:
		for e in l:
		    if type(e) is not type(""):
			print ":config_files is not a list of files."
		    elif not self.server.map_db_table.has_key(e):
			try:
			    new_db = TransducerMapDB(e)
			    self.server.map_db_table[e] = new_db
			except SyntaxError:
			    print "Encountered an error loading configuration file."
	if dict.has_key(":require_frame"):
	    self.conn.require_frame = 1
	else:
	    self.conn.require_frame = 0
	return None

    def Transduce(self, dict):
	if dict.has_key(":input_frame"):
	    map_dbs = []
	    if dict.has_key(":transductions"):
		for t in dict[":transductions"]:
		    try:
			map_dbs.append(self.server.map_db_table[t])
		    except:
			print "Couldn't find transduction %s." % t
			return {":error": "I heard your words, but I can't understand what you said because one of my servers is misconfigured."}
	    elif not self.server.default_map_db:
		print "No default transducer."
		return {":error": "I heard your words, but I can't understand what you said because one of my servers is misconfigured."}
	    else:
		print "Using default transducer."
		map_dbs.append(self.server.default_map_db)
	    out_frame = dict[":input_frame"]
	    verbose = dict.has_key(":transducer_debug") or \
		      self.server.transducer_verbose
	    for map_db in map_dbs:
		try:
		    out_frame = map_db.Transduce(out_frame, debug = verbose)
		except transducer_core.OperationError:		    
		    print "Encountered an error transducing the frame."
		    return {":error": "I heard your words, but I can't interpret them correctly because of an internal error."}
	    if self.conn.require_frame and \
	       ((Galaxy.GetObjectType(out_frame) is not Galaxy.GAL_FRAME) or \
		(out_frame.name == "__NOT_SET__")):
		return {":error": "I heard your words, but I can't understand them correctly."}
	    else:
		return {":output_frame": out_frame}
	else: return {":error": "I heard your words, but I can't understand them correctly because of an internal error."}

def main():
    s = TransducerServer(sys.argv, "transducer",
			 default_port = 19000,
			 env_class = TransducerEnvironment)
    s.AddDispatchFunction("reinitialize", TransducerEnvironment.Welcome)
    s.AddDispatchFunction("transduce", TransducerEnvironment.Transduce)
    s.RunServer()

main()
