# This file (c) Copyright 1998 - 2002 The MITRE Corporation
# 
# This file is part of the Galaxy Communicator system. It is licensed
# under the conditions described in the file LICENSE in the root 
# directory of the Galaxy Communicator system.

import string, types, sys, os

sys.path.insert(0, os.path.join(os.environ["GC_HOME"],
				"contrib", "MITRE", "templates"))

import GC_py_init

import Galaxy


TRANSDUCER_DIR = os.path.join(os.environ["GC_HOME"],
			      "contrib", "MITRE",
			      "servers", "src", "frame_transducer")

import parser

class TransducerMap:
    def __init__(self, name, env, fn_table):
	self.name = name
	self.fn_table = []
	for name, fn_name, code in fn_table:
	    exec code in env
	    self.fn_table.append(name, env[fn_name])

class TransducerEnvironment:
    def __init__(self, db):
	self.env = {}
	execfile(os.path.join(TRANSDUCER_DIR, "transducer_basics.py"),
		 self.env)
	if db.code:
	    exec db.code in self.env
    def Transduce(self, element, debug = 0):
	if Galaxy.GetObjectType(element) is Galaxy.GAL_FRAME:
	    element.parent = None
	    self.SetParentLinks(element)
	    target = Galaxy.Frame(name = "__NOT_SET__")
	    target.parent = None
	else:
	    target = None
	self.env["Target"] = target
	self.env["Source"] = element
	self.env["Debug"] = debug
	self.env["Altered"] = 0
	eval("Map('self', 'self', Any)", self.env)
	return self.env["Target"]    
    def SetParentLinks(self, frame):	
	self.SetParentListLinks(frame, frame.preds)
	for key, value in frame.items():
	    t = Galaxy.GetObjectType(value)
	    if t is Galaxy.GAL_FRAME:
		self.SetParentLinks(value)
	    elif t is Galaxy.GAL_LIST:
		self.SetParentListLinks(frame, value)
    def SetParentListLinks(self, parent, list):
	for elt in list:
	    t = Galaxy.GetObjectType(elt)
	    if t is Galaxy.GAL_FRAME:
		elt.parent = parent
		self.SetParentLinks(elt)
	    elif t is Galaxy.GAL_LIST:
		self.SetParentListLinks(parent, elt)

class TransducerMapDB:
    def __init__(self, file):
	# First, I want to load the file into an empty
	# dictionary, because that seems to be the only way
	# I can get intelligent syntax error messages.
	temp = {}
	execfile(file, temp)
	fp = open(file, "r")
	self.code = parser.compileast(parser.suite(fp.read()))
	fp.close()
    def Transduce(self, element, debug = 0):
	env = TransducerEnvironment(self)
	return env.Transduce(element, debug)
    
#
# This is the body of the operations.
# 

OperationError = "OperationError"

# I'm going to need to set the parent in order to
# process this.

# If we're setting a new value, we should allow 
# creation.

def CanonicalizeValue(v):
    t = Galaxy.GetObjectType(v)
    if t is Galaxy.GAL_SYMBOL:
	return v.str
    elif t is Galaxy.GAL_LIST:
	return map(CanonicalizeValue, v)
    else: return v

def ExtractCanonicalizedValue(path_obj, allow_create = 0):
    seed = path_obj.object
    path = path_obj.path_elements
    return _I_ExtractCanonicalizedValue(seed, path, allow_create)

# 3/28/00: Added a wildcard capability "*" for searching. 

def _I_ExtractCanonicalizedValue(seed, path, allow_create = 0):
    for i in range(len(path)):
	element = path[i]
	if element == "self":
	    pass
	elif element == "*":
	    # Wildcard search. Recurse on all key values and
	    # all preds, failing if there's an error. Check
	    # both the current list and the list following
	    # the wildcard (to cut off the recursion). The
	    # first element that works is returned.
	    cur_path = path[i:]
	    cur_path_suffix = path[i+1:]
	    if not cur_path_suffix:
		raise OperationError, "wildcard at path end"		
	    elif Galaxy.GetObjectType(seed) is Galaxy.GAL_LIST:
		for val in seed:
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path, allow_create)
		    except OperationError:
			pass
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path_suffix, allow_create)
		    except OperationError:
			pass
		# Final case: do nothing, like self.
	    elif Galaxy.GetObjectType(seed) is Galaxy.GAL_FRAME:
		for val in seed.values():
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path, allow_create)
		    except OperationError:
			pass
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path_suffix, allow_create)
		    except OperationError:
			pass
		for val in seed.preds:
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path, allow_create)
		    except OperationError:
			pass
		    try:
			return _I_ExtractCanonicalizedValue(val, cur_path_suffix, allow_create)
		    except OperationError:
			pass
		# Final case: do nothing, like self.
	elif type(element) is type(0):
	    # It's an index into a list.
	    if Galaxy.GetObjectType(seed) is not Galaxy.GAL_LIST:
		raise OperationError, "not a list"
	    try:
		seed = seed[element]
	    except IndexError, e:
		raise OperationError, e
	elif Galaxy.GetObjectType(seed) is not Galaxy.GAL_FRAME:
	    # This will cover the case where we got
	    # the preds, too.
	    raise OperationError, "not a frame"
	elif element == "frame_name":
	    seed = seed.name
	elif element == "parent":
	    seed = seed.parent
	elif element in [":preds", ":predicates"]:
	    if not seed.preds:
		raise OperationError, "no key"
	    seed = seed.preds
	elif element in [":pred", ":predicate"]:
	    if len(seed.preds) != 1:
		raise OperationError, "no key"
	    else:
		seed = seed.preds[0]
	elif not seed.has_key(element):
	    if allow_create:
		seed[element] = Galaxy.Frame("__NOT_SET__")
	        seed[element].parent = seed
		seed = seed[element]
	    else:
		raise OperationError, "no key"
	else:
	    seed = seed[element]
    return CanonicalizeValue(seed)

# Paths can be atoms or Path structures. Any element of a 
# path structure, or the atom itself, can be either self, 
# parent, frame_name or some other element, which is 
# taken to be a keyword. The result is a list, perhaps empty.
# Path can have Apply wrapped around it, in which case the
# operation is applied later.

# The map will now be a list.

# So we have a path. How should the path be interpreted?
# If the last element of the path is self or parent, do a 
# merge. Otherwise, do a set. So the algorithm is to
# take the path besides the last element and compute the 
# place to put something. Remember, every path is interpreted
# as if it had "self" at the beginning.

# If the subpath is empty and the final element
# is self and the value is not a frame, then
# we're promoting. So this function should return the value
# we're supposed to return. In all cases besides this one,
# it should be the input frame.

# Also, as we construct the target frame, we want to 
# make sure that the parent links are established.

def SetCanonicalizedValue(path_obj, val, do_append = 0):
    frame = path_obj.object
    path = path_obj.path_elements
    return _I_SetCanonicalizedValue(frame, path, val, do_append)

def _I_SetCanonicalizedValue(frame, path, val, do_append = 0):
    # Path has been canonicalized.    
    sub_path = path[:-1]    
    final_element = path[-1]
    
    # If the path is empty and the final element
    # is self and the value is not a frame, then
    # we're promoting.
    if len(sub_path) == 0 and \
       final_element == "self" and \
       Galaxy.GetObjectType(val) is not Galaxy.GAL_FRAME:
	if do_append:
	    # Can't really do this.
	    return frame, 0
	else:
	    return val, 0
    
    # Now, normal processing. First, extract the value
    # using the subpath.
    try:
	target_frame = _I_ExtractCanonicalizedValue(frame, sub_path,
						    allow_create = 1)
    except OperationError:
	return frame, 0
    
    # The subpath better be a frame...
    if Galaxy.GetObjectType(target_frame) is not Galaxy.GAL_FRAME:
	return frame, 0

    # We have special keys: self, parent, frame_name.    
    # We have to worry about pred here too.

    altered = 0
    if type(final_element) is type(0):
	# if it's an index
	if Galaxy.GetObjectType(target_frame) is not Galaxy.GAL_LIST:
	    # Don't do anything.
	    return frame, altered
	if len(target_frame) < (final_element + 1):
	    return frame, altered
	if not do_append:
	    target_frame[final_element] = val
	    altered = 1
	elif Galaxy.GetObjectType(target_frame[final_element]) is not Galaxy.GAL_LIST:
	    return frame, altered
	else:
	    target_frame[final_element].append(val)
	    altered = 1
    elif final_element == "*":
	# Can't set wildcard.
	return frame, altered
    elif final_element in ["self", "parent"]:
	if Galaxy.GetObjectType(val) is not Galaxy.GAL_FRAME:
	    # Don't do anything.
	    return frame, altered
	if do_append:
	    # Can't merge and append.
	    return frame, altered
	if final_element == "parent":
	    target_frame = target_frame.parent
	    altered = 1
	# Merge.
	for key in val.keys():
	    target_frame[key] = val[key]
	    altered = 1
	if val.name != "__NOT_SET__":
	    target_frame.name = val.name
	    altered = 1
	if val.preds:
	    target_frame.preds = target_frame.preds + val.preds
	    altered = 1
    elif final_element == "frame_name":
	if not do_append:
	    # Can't append to the frame name.
	    target_frame.name = val
	    altered = 1
    elif final_element in [":preds", ":predicates"]:	
	if do_append:
	    if Galaxy.GetObjectType(val) is not Galaxy.GAL_FRAME:
		return target_frame, altered
	    val = val.Copy()
	    val.parent = target_frame
	    target_frame.preds.append(val)
	    altered = 1
	elif Galaxy.GetObjectType(val) is not Galaxy.GAL_LIST:
	    # Quietly fail.
	    return target_frame, altered
	else:
	    newval = []
	    for elt in val:
		if Galaxy.GetObjectType(elt) is not Galaxy.GAL_FRAME:
		    return target_frame, altered
		newelt = elt.Copy()
		newelt.parent = target_frame
		newval.append(newelt)
	    target_frame.preds = newval
	    altered = 1
    elif final_element in [":pred", ":predicate"]:
	if Galaxy.GetObjectType(val) is Galaxy.GAL_FRAME:
	    val = val.Copy()
	    val.parent = target_frame
	if not do_append:
	    target_frame.preds = [val]
	    altered = 1
	else:
	    target_frame.preds.append(val)
	    altered = 1
    else:
	if Galaxy.GetObjectType(val) is Galaxy.GAL_FRAME:
	    val = val.Copy()
	    val.parent = target_frame
	if not do_append:
	    target_frame[final_element] = val
	    altered = 1
	elif not target_frame.has_key(final_element):
	    target_frame[final_element] = [val]
	    altered = 1
	elif Galaxy.GetObjectType(target_frame[final_element]) is not Galaxy.GAL_LIST:
	    return frame, altered
	else:
	    target_frame[final_element].append(val)
	    altered = 1
    return frame, altered
