/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

package org.mitre.galaxycommunicator.visualizer;

/**
 * A simple interface for "debugging" Hub programs. 
 *
 * @author Steve Wohlever, MITRE Corporation, wohlever@mitre.org
 */
public class DebugUI extends javax.swing.JFrame
{ 

    /** The server that is associated with the parent UI. */
    private VisualizerServer server = null;

    /** 
     * The environment (from the server) that the UI will use to send the 
     * next debug command.
     */
    private galaxy.server.Environment env = null;

    private javax.swing.JButton stepButton = null;
    private javax.swing.JButton continueButton = null; 
    private javax.swing.JButton stopButton = null;
    private javax.swing.JButton sessionDBButton = null;
    private javax.swing.JButton globalsButton = null;
    private javax.swing.JButton locksButton = null;
    private javax.swing.JButton messageButton = null;
    private javax.swing.JButton serverButton = null;
    private javax.swing.JButton sessionButton = null;
    private javax.swing.JButton tokenButton = null;
    private final int NUM_BUTTONS = 10;

    private java.util.List buttonList = null;
    private java.util.Map buttonCmdMap = null;

    private javax.swing.JPanel buttonPanel = null;

    private javax.swing.JTextArea textArea = null;
    private final int TEXT_ROWS = 10;
    private final int TEXT_COLS = 20;

    /**
     * Constructor.
     */
    public DebugUI()
    {
	super("Debugger");

	buttonPanel = new javax.swing.JPanel(new java.awt.GridLayout(0, 2));

	buttonList = new java.util.ArrayList(NUM_BUTTONS);
	buttonCmdMap = new java.util.HashMap(NUM_BUTTONS);

	stepButton = createButton("Step", "c", "Take one step through program.", buttonPanel);
	continueButton = createButton("Continue", "C", "Disable debugging and continue the program.", buttonPanel);
	stopButton = createButton("Stop Hub", "e", "Stops the Hub.", buttonPanel);
	sessionDBButton = createButton("Session DB", "d", "Display the session database.", buttonPanel);

	globalsButton = createButton("Globals", "g", "Display global settings.", buttonPanel);
	locksButton = createButton("Locks", "l", "Display lock information.", buttonPanel);
	messageButton = createButton("Message", "m", "Display the current message.", buttonPanel);
	serverButton = createButton("Server", "r", "Display server information.", buttonPanel);
	sessionButton = createButton("Session", "s", "Display information on the current session.", buttonPanel);
	tokenButton = createButton("Token", "t", "Display information on the current token.", buttonPanel);

	textArea = new javax.swing.JTextArea(TEXT_ROWS, TEXT_COLS);
	textArea.setEditable(false);
	javax.swing.JScrollPane scrollPane = new javax.swing.JScrollPane(textArea);

	java.awt.Container contentPane = getContentPane();
	contentPane.add(scrollPane, java.awt.BorderLayout.CENTER);
	contentPane.add(buttonPanel, java.awt.BorderLayout.SOUTH);
	pack();
    }

    /**
     * Creates a debug button
     *
     * @param label the button label
     * @param cmd the debug command string associated with the button
     * @param toolTip the tool tip associated with the button
     * @param panel the panel to which the button should be added
     *
     * @return the button
     */
    private javax.swing.JButton createButton(java.lang.String label,
					     java.lang.String cmd,
					     java.lang.String toolTip,
					     javax.swing.JPanel panel)
    {
	javax.swing.JButton button = new javax.swing.JButton(label);
	buttonList.add(button);
	buttonCmdMap.put(button, cmd);
	button.setToolTipText(toolTip);
	button.addActionListener(new java.awt.event.ActionListener()  {
		public void actionPerformed(java.awt.event.ActionEvent event) {
		    debugActionPerformed(event);
		}
	    });
	panel.add(button);
	return button;
    }

    /**
     * Event handler for debug buttons.
     *
     * @param event the event to handle
     */
    public void debugActionPerformed(java.awt.event.ActionEvent event)
    {
	textArea.setText(null);
	if(server != null && env != null) {
	    java.lang.Object source = event.getSource();
	    java.lang.String cmd = (String) buttonCmdMap.get(source);
	    if(cmd != null) {

		// Deal with special cases first.
		if(cmd.equalsIgnoreCase("c"))
		    setVisible(false);
		
		server.sendDebugCommand(env, cmd);
		env = null;
	    }
	}
    }

    /** 
     * Set the server.
     *
     * @param server the server
     */
    protected void setServer(VisualizerServer server)
    {
	this.server = server;
    }   

    /**
     * Sets the environment that the UI will use to send the next debug
     * command.
     *
     * @param env the environment
     */
    protected void setEnv(galaxy.server.Environment env)
    {
	this.env = env;
    }

    /**
     * Sets the text in the UI's text area.
     *
     * @param text the text
     */
    protected void setText(java.lang.String text)
    {
	textArea.setText(text);
    }
}

