/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

package org.mitre.galaxycommunicator.visualizer;

/**
 * A simple interface for configuring the Hub.
 *
 * @author Steve Wohlever, MITRE Corporation, wohlever@mitre.org
 */
public class HubFrame extends javax.swing.JFrame implements java.awt.event.ActionListener, javax.swing.event.ChangeListener
{      
    private javax.swing.JCheckBox debugCheckBox = null;
    private javax.swing.JCheckBox validateCheckBox = null;
    private javax.swing.JSlider verbositySlider = null;

    private javax.swing.JTextField initTextField = null;
    private javax.swing.JTextField locationsTextField = null;

    private final int NUM_TEXT_FIELD_COLS = 1;
    private final int DEFAULT_HUB_VERBOSITY = 3;

    private org.mitre.galaxycommunicator.visualizer.VisualizerProperties props = null;
    /**
     * Constructor.
     */
    public HubFrame(org.mitre.galaxycommunicator.visualizer.VisualizerProperties props)
    {
	super("Hub Configuration");

	this.props = props;

	// Configure the frame.
	java.awt.Container contentPane = getContentPane();
	contentPane.setLayout(new java.awt.BorderLayout());
	setResizable(false);

	// Set up a border.
	javax.swing.border.Border border = javax.swing.BorderFactory.createLoweredBevelBorder();

	// Build the check box panel.
	javax.swing.JPanel checkBoxPanel = new javax.swing.JPanel();
	checkBoxPanel.setBorder(border);
	checkBoxPanel.setLayout(new javax.swing.BoxLayout(checkBoxPanel, javax.swing.BoxLayout.X_AXIS));
	debugCheckBox = new javax.swing.JCheckBox("Debug enabled");
	debugCheckBox.addActionListener(this);
	debugCheckBox.setSelected(false);
	validateCheckBox = new javax.swing.JCheckBox("Validation enabled");
	validateCheckBox.addActionListener(this);
	validateCheckBox.setSelected(false);
	checkBoxPanel.add(debugCheckBox);
	checkBoxPanel.add(validateCheckBox);
	
	// Build the verbosity panel.
	javax.swing.JPanel verbosityPanel = new javax.swing.JPanel();
	verbosityPanel.setBorder(border);
	verbosityPanel.setLayout(new javax.swing.BoxLayout(verbosityPanel, javax.swing.BoxLayout.X_AXIS));
	verbositySlider = new javax.swing.JSlider(galaxy.server.MainServer.MIN_VERBOSITY_LEVEL, galaxy.server.MainServer.MAX_VERBOSITY_LEVEL, DEFAULT_HUB_VERBOSITY);
	verbositySlider.addChangeListener(this);
	verbositySlider.setMajorTickSpacing(1);
	verbositySlider.setSnapToTicks(true);
	verbositySlider.setPaintTicks(true);
	verbositySlider.setPaintLabels(true);	
	javax.swing.JLabel verbosityLabel = new javax.swing.JLabel(" -verbosity");
	verbosityPanel.add(verbosityLabel);
	verbosityPanel.add(verbositySlider);

	// Build the init panel.
	javax.swing.JPanel initPanel = new javax.swing.JPanel();
	initPanel.setLayout(new javax.swing.BoxLayout(initPanel, javax.swing.BoxLayout.X_AXIS));
	javax.swing.JLabel initLabel = new javax.swing.JLabel(" -init");
	initTextField = new javax.swing.JTextField(NUM_TEXT_FIELD_COLS);
	initTextField.addActionListener(this);
	initPanel.add(initLabel);
	initPanel.add(initTextField);

	// Build the -locations panel.
	javax.swing.JPanel locationsPanel = new javax.swing.JPanel();
	locationsPanel.setLayout(new javax.swing.BoxLayout(locationsPanel, javax.swing.BoxLayout.X_AXIS));
	javax.swing.JLabel locationsLabel = new javax.swing.JLabel(" -locations");
	locationsTextField = new javax.swing.JTextField(NUM_TEXT_FIELD_COLS);
	locationsTextField.addActionListener(this);
	locationsPanel.add(locationsLabel);
	locationsPanel.add(locationsTextField);

	// Build the string argument panel.
	javax.swing.JPanel stringArgPanel = new javax.swing.JPanel();
	stringArgPanel.setBorder(border);
	stringArgPanel.setLayout(new java.awt.BorderLayout());
	stringArgPanel.add(initPanel, java.awt.BorderLayout.NORTH);
	stringArgPanel.add(locationsPanel, java.awt.BorderLayout.SOUTH);
	
	// Create the close button.
	javax.swing.JButton closeButton = new javax.swing.JButton("Close");
	closeButton.addActionListener(new java.awt.event.ActionListener()  {
		public void actionPerformed(java.awt.event.ActionEvent event) {
		    closeActionPerformed(event);
		}
	    });
	closeButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);

	// Build the config panel.
	javax.swing.JPanel configPanel = new javax.swing.JPanel();
	configPanel.setLayout(new java.awt.BorderLayout());
	configPanel.add(checkBoxPanel, java.awt.BorderLayout.NORTH);
	configPanel.add(verbosityPanel, java.awt.BorderLayout.CENTER);
	configPanel.add(stringArgPanel, java.awt.BorderLayout.SOUTH);

	// Finally, put it all together.
	contentPane.add(configPanel, java.awt.BorderLayout.CENTER);
	contentPane.add(closeButton, java.awt.BorderLayout.SOUTH);

	pack();
    }

    public void closeActionPerformed(java.awt.event.ActionEvent event)
    {
	// Just in case there were changes to these fields that were not
	// caught.
	props.setInit(getInitValues());
	props.setLocations(getLocations());	

	setVisible(false);
    }

    // Gettor and settor methods

    protected boolean debugEnabled()
    {
	return debugCheckBox.isSelected();
    }

    protected boolean validateEnabled()
    {
	return validateCheckBox.isSelected();
    }

    protected int getVerbosity()
    {
	return verbositySlider.getValue();
    }

    protected java.lang.String getInitValues()
    {
	return initTextField.getText();
    }

    protected java.lang.String getLocations()
    {
	return locationsTextField.getText();
    }

    protected void setDebugEnabled(boolean enabled)
    {
	debugCheckBox.setSelected(enabled);
    }

    protected void setValidateEnabled(boolean enabled)
    {
	validateCheckBox.setSelected(enabled);
    }
    
    protected void setVerbosity(int verbosity)
    {
	if(verbosity >= galaxy.server.MainServer.MIN_VERBOSITY_LEVEL &&
	   verbosity <= galaxy.server.MainServer.MAX_VERBOSITY_LEVEL)
	    verbositySlider.setValue(verbosity);
    }

    protected void setInitValues(java.lang.String initValues)
    {
	initTextField.setText(initValues);
    }

    protected void setLocations(java.lang.String locations)
    {
	locationsTextField.setText(locations);
    }

    public void setEnabled(boolean enabled)
    {
	super.setEnabled(enabled);
	debugCheckBox.setEnabled(enabled);
	validateCheckBox.setEnabled(enabled);
	verbositySlider.setEnabled(enabled);
	initTextField.setEnabled(enabled);
	locationsTextField.setEnabled(enabled);
    }

    public void actionPerformed(java.awt.event.ActionEvent event)
    { 
	// TODO: Need a better way to track changes to text fields.

	java.lang.Object source = event.getSource();
	if(source.equals(debugCheckBox)) {
	    props.setDebug(debugEnabled());
	} else if(source.equals(validateCheckBox)) {
	    props.setValidate(validateEnabled());
	} else if(source.equals(initTextField)) {
	    props.setInit(getInitValues()); // Only occurs if user presses <Return> after modifying the field
	} else if(source.equals(locationsTextField)) {
	    props.setLocations(getLocations()); // Only occurs if user presses <Return> after modifying the field
	}
    }

    public void stateChanged(javax.swing.event.ChangeEvent event)
    {
	if(event.getSource().equals(verbositySlider)) {
	    props.setHubVerbosity(getVerbosity());
	}
    }
}

