/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

package org.mitre.galaxycommunicator.visualizer;

import java.util.List;
import java.awt.Color;

/** 
 * Encapsulates information about a service provider.
 *
 * @author Steve Wohlever, MITRE Corporation, wohlever@mitre.org
 */
public class ProviderInfo
{
    /** A unique id for this provider. */
    private String id = null;
    
    /** The UI label for this provider. */
    private String label = null;

    /** The file name of the icon used to represent this provider in the UI. */
    private String iconFileName = null;

    /** Collection of popup menu items associated with this provider. */
    private List menuItems;

    /** If true, the Hub contacts this provider.  */
    private boolean hubIsClient = true;

    /** If true, the associated provider is avaiable for connection. */
    private boolean available = false;
    
    /** If true, the associated provider is connected. */
    private boolean connected = false;

    /** If true, this provider has been initialized by the GUI (e.g., associated icon image and menu items set up). */
    private boolean initialized = false;
    
    public ProviderInfo(String id, String label, String iconFileName,
			List menuItems, boolean hubIsClient,
			boolean available, boolean connected)
    {
	this.id = id;
	this.label = label;
	this.iconFileName = iconFileName;
	this.menuItems = menuItems;
	this.hubIsClient = hubIsClient;
	this.available = available;
	this.connected = connected;
    }
    
    public String getId()
    {
	return id;
    }

    public String getLabel()
    {
	return label;
    }
   
    public void setIconFileName(String iconFileName)
    {
	this.iconFileName = iconFileName;
    }

    public String getIconFileName()
    {
	return iconFileName;
    }
    
    public List getMenuItems()
    {
	return menuItems;
    }

    public boolean isHubTheClient()
    {
	return hubIsClient;
    }

    public void setAvailabilityStatus(boolean available)
    {
	this.available = available;
    }

    public void setConnectionStatus(boolean connected)
    {
	this.connected = connected;
    }

    public boolean isAvailable()
    {
	return available;
    }

    public boolean isConnected()
    {
	return connected;
    }

    public void setInitialized(boolean initialized)
    {
	this.initialized = initialized;
    }

    public boolean isInitialized()
    {
	return initialized;
    }

}
