/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

package org.mitre.galaxycommunicator.visualizer;

import galaxy.server.MainServer;
import galaxy.server.ServerArgParser;

/**
 * The main Communicator visualizer server.
 *
 * @author Steve Wohlever, MITRE Corporation, wohlever@mitre.org
 */
public class VisualizerMainServer extends galaxy.server.MainServer 
{
    private VisualizerUI ui;

    public VisualizerMainServer(String name, String args[]) 
    {
	super(name, args, "-hub <Hub executable>            The path to the Hub to use.\n" +
	                  "-pgm_file <file name>            Hub program file to use.\n" +
	                  "-locations <server info>         The Hub -locations argument.\n" +
                          "-init <init info>                The Hub -init argument.\n" +
	                  "-debug                           Start the Hub in debug mode.\n" +
                          "-hub_verbosity <verbosity level> The Hub verbosity level.\n" +
	                  "-props <file name>               Custom properties file to use.\n" +
	                  "-use_hub_output_dialog           Indicates that a separate dialog\n" +
                          "                                 window should be used to display\n" +
                          "                                 normal Hub output.\n" +
	                  "-use_hub_status_dialog           Indicates that a separate dialog\n" +
                          "                                 window should be used to display\n" +
	                  "                                 Hub status output.\n");
    }

    public void setUI(VisualizerUI ui)
    {
	this.ui = ui;
    }

    public VisualizerUI getUI()
    {
	return ui;
    }

    public ServerArgParser getArgParser()
    {
	return argParser;
    }

    //=========================================================================
    // Main routine
    //=========================================================================
    
    public static void main(String[] args)
    { 
	try {
	    VisualizerMainServer mainServer = 
		new VisualizerMainServer("visualizer", args);
	    VisualizerUI ui = new VisualizerUI("Galaxy Communicator Visualizer", 
					       mainServer.getArgParser(),
					       mainServer.getPort());
	    mainServer.setUI(ui);
	    mainServer.setServerClassName("org.mitre.galaxycommunicator.visualizer.VisualizerServer");
	    mainServer.start();
	} catch(Exception ex) {
	    System.out.println(ex.toString());
	    ex.printStackTrace();
	    System.exit(1);
	}
    }
}
