/*
  This file (c) Copyright 1998 - 2002 The MITRE Corporation
  
  This file is part of the Galaxy Communicator system. It is licensed
  under the conditions described in the file LICENSE in the root 
  directory of the Galaxy Communicator system.
*/

package org.mitre.galaxycommunicator.visualizer;

/**
 * This class encapsulates the properties that define the visualizer
 * GUI and runtime.
 */
public class VisualizerProperties
{
    // Set up the color map.
    private static java.util.Map colorMap = new java.util.HashMap(13);
    static {
	colorMap.put("black", java.awt.Color.black);
	colorMap.put("blue", java.awt.Color.blue);
	colorMap.put("cyan", java.awt.Color.cyan);
	colorMap.put("darkgray", java.awt.Color.darkGray);
	colorMap.put("gray", java.awt.Color.gray);
	colorMap.put("green", java.awt.Color.green);
	colorMap.put("lightgray", java.awt.Color.lightGray);
	colorMap.put("magenta", java.awt.Color.magenta);
	colorMap.put("orange", java.awt.Color.orange);
	colorMap.put("pink", java.awt.Color.pink);
	colorMap.put("red", java.awt.Color.red);
	colorMap.put("white", java.awt.Color.white);
	colorMap.put("yellow", java.awt.Color.yellow);
    }

    private java.util.Properties props = null;
    private final java.lang.String DEFAULT_PROPERTIES_FILE = "org/mitre/galaxycommunicator/visualizer/defaults.properties";

    private int drawingAreaWidth = 700;
    private int drawingAreaHeight = 500;
    private int providerRadius = 10;
    private java.awt.Color providerColor = java.awt.Color.blue;
    private java.awt.Color providerHighlightColor = java.awt.Color.red;
    private int providerFlashMsec = 75 ; 
    private java.awt.Color unavailableColor = java.awt.Color.lightGray;
    private int hubRadius = 40;
    private java.awt.Color hubRunningColor = new java.awt.Color(0,180,0);
    private java.awt.Color hubStoppedColor = java.awt.Color.red; 
    private int linkLineWidth = 3;
    private int linkLineLength = 200;
    private java.awt.Color linkLineColor = java.awt.Color.black;
    private int messageTrafficLineWidth = 3;
    private java.awt.Color messageTrafficToProviderLineColor = new java.awt.Color(0,180,0);
    private java.awt.Color messageTrafficToHubLineColor = java.awt.Color.blue;
    private int arrowheadWidth = 10;
    private int arrowheadHeight = 10;
    private int arrowFlashMsec = 150;
    private int labelFontSize = 18;
    private java.awt.Color labelForegroundColor = java.awt.Color.black;
    private java.awt.Color labelBackgroundColor = java.awt.Color.yellow;
    private boolean useHubOutputDialog = false;
    private boolean useHubStatusDialog = false;

    private java.lang.String hubPath = "hub";
    private java.lang.String programFile = null;
    private java.lang.String serverLocationFile = null;
    private java.lang.String locations = null;
    private java.lang.String init = null;
    private boolean debug = false;
    private boolean validate = false;
    private int hubVerbosity = 3;

    private java.lang.String demoHomePath = ".";
    private java.lang.String demoHomePrefix = "DEMO_HOME";

    private galaxy.util.Logger logger = null;
    private final int LOGGING_LEVEL = galaxy.server.MainServer.INITIALIZATION_DETAILS_VERBOSITY_LEVEL;

    public VisualizerProperties(java.lang.String propsFileName) throws Exception
    {
	this();
	if(propsFileName != null)
	    loadProperties(propsFileName);
    }
    
    public VisualizerProperties() throws Exception
    {
	logger = galaxy.util.Logger.getLogger();
	props = new java.util.Properties();
	loadProperties(DEFAULT_PROPERTIES_FILE);
    }

    private void loadProperties(java.lang.String propsFileName) 
	throws java.lang.Exception
    {
	logger.logMessage("\n\nProcessing properties file " + propsFileName + "...", LOGGING_LEVEL);
	java.io.InputStream in = null;

	// Try to find the file in the class path.
	in = this.getClass().getClassLoader().getResourceAsStream(propsFileName);
	if(in == null) {
	    logger.logMessage("Can't file properties file " + propsFileName + " in class path, looking for it in file system...", LOGGING_LEVEL);

	    // If that fails, try to load the file based solely on its name.
	    in = new java.io.FileInputStream(propsFileName);
	}
	
	if(in != null) {
	    props.load(in);
	    if(galaxy.server.MainServer.getVerbosityLevel() >= LOGGING_LEVEL) {
		java.lang.System.out.println("=== Properties  from " + propsFileName + " ===");
		props.list(java.lang.System.out);
		java.lang.System.out.println("\n");
	    }

	    in.close();
	} else {
	    throw new Exception("Could not open properties file \"" + propsFileName + "\".");
	}

	processProperties();
    }

    private void processProperties()
    {	
	java.lang.String propValue;
	
	propValue = props.getProperty("drawingAreaWidth");
	if(propValue != null) {
	    drawingAreaWidth = getIntValue(propValue, drawingAreaWidth);
	    logger.logMessage("Property drawingAreaWidth = " + drawingAreaWidth, LOGGING_LEVEL);
	}

	propValue = props.getProperty("drawingAreaHeight");
	if(propValue != null) {
	    drawingAreaHeight = getIntValue(propValue, drawingAreaHeight);
	    logger.logMessage("Property drawingAreaHeight = " + drawingAreaHeight, LOGGING_LEVEL);
	}

	propValue = props.getProperty("providerRadius");
	if(propValue != null) {
	    providerRadius = getIntValue(propValue, providerRadius);
	    logger.logMessage("Property providerRadius = " + providerRadius, LOGGING_LEVEL);
	}

	propValue = props.getProperty("providerColor");
	if(propValue != null) {
	    providerColor = getColorValue(propValue, providerColor);
	    logger.logMessage("Property providerColor = " + providerColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("providerHighlightColor");
	if(propValue != null) {
	    providerHighlightColor = getColorValue(propValue, providerHighlightColor);
	    logger.logMessage("Property providerHighlightColor = " + providerHighlightColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("providerFlashMsec");
	if(propValue != null) {
	    providerFlashMsec= getIntValue(propValue, providerFlashMsec);
	    logger.logMessage("Property providerFlashMsec = " + providerFlashMsec, LOGGING_LEVEL);
	}

	propValue = props.getProperty("unavailableColor");
	if(propValue != null) {
	    unavailableColor = getColorValue(propValue, unavailableColor);
	    logger.logMessage("Property unavailableColor = " + unavailableColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("hubRadius");
	if(propValue != null) {
	    hubRadius= getIntValue(propValue, hubRadius);
	    logger.logMessage("Property hubRadius = " + hubRadius, LOGGING_LEVEL);
	}

	propValue = props.getProperty("hubRunningColor");
	if(propValue != null) {
	    hubRunningColor = getColorValue(propValue, hubRunningColor);
	    logger.logMessage("Property hubRunningColor = " + hubRunningColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("hubStoppedColor");
	if(propValue != null) {
	    hubStoppedColor = getColorValue(propValue, hubStoppedColor);
	    logger.logMessage("Property hubStoppedColor = " + hubStoppedColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("linkLineWidth");
	if(propValue != null) {
	    linkLineWidth= getIntValue(propValue, linkLineWidth);
	    logger.logMessage("Property linkLineWidth = " + linkLineWidth, LOGGING_LEVEL);
	}

	propValue = props.getProperty("linkLineLength");
	if(propValue != null) {
	    linkLineLength = getIntValue(propValue, linkLineLength);
	    logger.logMessage("Property linkLineLength = " + linkLineLength, LOGGING_LEVEL);
	}

	propValue = props.getProperty("linkLineColor");
	if(propValue != null) {
	    linkLineColor = getColorValue(propValue, linkLineColor);
	    logger.logMessage("Property linkLineColor = " + linkLineColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("messageTrafficLineWidth");
	if(propValue != null) {
	    messageTrafficLineWidth = getIntValue(propValue, messageTrafficLineWidth);
	    logger.logMessage("Property messageTrafficLineWidth = " + messageTrafficLineWidth, LOGGING_LEVEL);
	}

	propValue = props.getProperty("messageTrafficToProviderLineColor");
	if(propValue != null) {
	    messageTrafficToProviderLineColor = getColorValue(propValue, messageTrafficToProviderLineColor);
	    logger.logMessage("Property messageTrafficToProviderLineColor = " + messageTrafficToProviderLineColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("messageTrafficToHubLineColor");
	if(propValue != null) {
	    messageTrafficToHubLineColor = getColorValue(propValue, messageTrafficToHubLineColor);
	    logger.logMessage("Property messageTrafficToHubLineColor = " + messageTrafficToHubLineColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("arrowheadWidth");
	if(propValue != null) {
	    arrowheadWidth = getIntValue(propValue, arrowheadWidth);
	    logger.logMessage("Property arrowheadWidth = " + arrowheadWidth, LOGGING_LEVEL);
	}

	propValue = props.getProperty("arrowheadHeight");
	if(propValue != null) {
	    arrowheadHeight = getIntValue(propValue, arrowheadHeight);
	    logger.logMessage("Property arrowheadHeight = " + arrowheadHeight, LOGGING_LEVEL);
	}

	propValue = props.getProperty("arrowFlashMsec");
	if(propValue != null) {
	    arrowFlashMsec = getIntValue(propValue, arrowFlashMsec);
	    logger.logMessage("Property arrowFlashMsec = " + arrowFlashMsec, LOGGING_LEVEL);
	}

	propValue = props.getProperty("labelFontSize");
	if(propValue != null) {
	    labelFontSize = getIntValue(propValue, labelFontSize);
	    logger.logMessage("Property labelFontSize = " + labelFontSize, LOGGING_LEVEL);
	}

	propValue = props.getProperty("labelForegroundColor");
	if(propValue != null) {
	    labelForegroundColor = getColorValue(propValue, labelForegroundColor);
	    logger.logMessage("Property labelForegroundColor = " + labelForegroundColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("labelBackgroundColor");
	if(propValue != null) {
	    labelBackgroundColor = getColorValue(propValue, labelBackgroundColor);
	    logger.logMessage("Property labelBackgroundColor = " + labelBackgroundColor, LOGGING_LEVEL);
	}

	propValue = props.getProperty("labelBackgroundColor");
	if(propValue != null) {
	    labelBackgroundColor = getColorValue(propValue, labelBackgroundColor);
	    logger.logMessage("Property labelBackgroundColor = " + labelBackgroundColor, LOGGING_LEVEL);
	}
	
	propValue = props.getProperty("useHubOutputDialog");
	if(propValue != null) {
	    if(propValue.trim().equalsIgnoreCase("true"))
		useHubOutputDialog = true;
	    else if(propValue.trim().equalsIgnoreCase("false"))
		useHubOutputDialog = false;
	    logger.logMessage("Property useHubOutputDialog = " + useHubOutputDialog, LOGGING_LEVEL);
	}

	propValue = props.getProperty("useHubStatusDialog");
	if(propValue != null) {
	    if(propValue.trim().equalsIgnoreCase("true"))
		useHubStatusDialog = true;
	    else if(propValue.trim().equalsIgnoreCase("false"))
		useHubStatusDialog = false;
	    logger.logMessage("Property useHubStatusDialog = " + useHubStatusDialog, LOGGING_LEVEL);
	}
	
	propValue = props.getProperty("hubPath");
	if(propValue != null) {
	    hubPath = propValue.trim();
	    if(hubPath.equalsIgnoreCase("null"))
		hubPath = "";
	    logger.logMessage("Property hubPath = " + hubPath, LOGGING_LEVEL);
	}

	propValue = props.getProperty("programFile");
	if(propValue != null) {
	    programFile = propValue.trim();
	    if(programFile.equalsIgnoreCase("null"))
		programFile = "";
	    logger.logMessage("Property programFile = " + programFile, LOGGING_LEVEL);
	}

	propValue = props.getProperty("serverLocationFile");
	if(propValue != null) {
	    serverLocationFile = propValue.trim();
	    if(serverLocationFile.equalsIgnoreCase("null"))
		serverLocationFile = "";
	    logger.logMessage("Property serverLocationFile = " + serverLocationFile, LOGGING_LEVEL);
	}

	propValue = props.getProperty("locations");
	if(propValue != null) {
	    locations = propValue.trim();
	    if(locations.equalsIgnoreCase("null"))
		locations = "";
	    logger.logMessage("Property locations = " + locations, LOGGING_LEVEL);
	}

	propValue = props.getProperty("init");
	if(propValue != null) {
	    init = propValue.trim();
	    if(init.equalsIgnoreCase("null"))
		init = "";
	    logger.logMessage("Property init = " + init, LOGGING_LEVEL);
	}

	propValue = props.getProperty("debug");
	if(propValue != null) {
	    if(propValue.trim().equalsIgnoreCase("true"))
		debug = true;
	    else if(propValue.trim().equalsIgnoreCase("false"))
		debug = false;
	    logger.logMessage("Property debug = " + debug, LOGGING_LEVEL);
	}

	propValue = props.getProperty("validate");
	if(propValue != null) {
	    if(propValue.trim().equalsIgnoreCase("true"))
		validate = true;
	    else if(propValue.trim().equalsIgnoreCase("false"))
		validate = false;
	    logger.logMessage("Property validate = " + validate, LOGGING_LEVEL);
	}

	propValue = props.getProperty("hubVerbosity");
	if(propValue != null) {
	    hubVerbosity = getIntValue(propValue, hubVerbosity);
	    logger.logMessage("Property hubVerbosity = " + hubVerbosity, LOGGING_LEVEL);
	}

	propValue = props.getProperty("demoHomePath");
	if(propValue != null) {
	    demoHomePath = propValue.trim();
	    if(demoHomePath.equalsIgnoreCase("null"))
		demoHomePath = "";
	    logger.logMessage("Property demoHomePath = " + demoHomePath, LOGGING_LEVEL);
	}

	propValue = props.getProperty("demoHomePrefix");
	if(propValue != null) {
	    demoHomePrefix = propValue.trim();
	    if(demoHomePrefix.equalsIgnoreCase("null"))
		demoHomePrefix = "";
	    logger.logMessage("Property demoHomePrefix = " + demoHomePrefix, LOGGING_LEVEL);
	}
    }
    
    private int getIntValue(java.lang.String propValue, int defaultValue)
    {
	try {
	    Integer integerValue = Integer.decode(propValue.trim());
	    if(integerValue.intValue() > 0)
		return integerValue.intValue();
	} catch(Exception ex) {
	} 
	return defaultValue;
    }
    
    private java.awt.Color getColorValue(java.lang.String propValue, java.awt.Color defaultValue)
    {
	propValue = propValue.trim();
	java.awt.Color color = (java.awt.Color) VisualizerProperties.colorMap.get(propValue);
	if(color != null)
	    return color;

	java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(propValue, ",");
	java.lang.String token;
	int[] colorComponents = new int[3];
	int count = 0;
	while(tokenizer.hasMoreTokens()) {
	    token = tokenizer.nextToken();
	    try {
		colorComponents[count] = Integer.parseInt(token);
	    } catch(java.lang.NumberFormatException nfex) {
		return defaultValue;
	    }
	    ++count;
	}
	
	if(count != 3) {
	    return defaultValue;
	}

	return new java.awt.Color(colorComponents[0], colorComponents[1], colorComponents[2]);
    } 

    public int getDrawingAreaWidth()
    {
	return drawingAreaWidth;
    }

    public int getDrawingAreaHeight()
    {
	return drawingAreaHeight;
    }

    public int getProviderRadius()
    {
	return providerRadius;
    }

    public java.awt.Color getProviderColor()
    {
	return providerColor;
    }

    public java.awt.Color getProviderHighlightColor()
    {
	return providerHighlightColor;
    }

    public int getProviderFlashMsec()
    {
	return providerFlashMsec;
    }

    public java.awt.Color getUnavailableColor()
    {
	return unavailableColor;
    }

    public int getHubRadius()
    {
	return hubRadius;
    }

    public java.awt.Color getHubRunningColor()
    {
	return hubRunningColor;
    }

    public java.awt.Color getHubStoppedColor()
    {
	return hubStoppedColor;
    }

    public int getLinkLineWidth()
    {
	return linkLineWidth;
    }

    public int getLinkLineLength()
    {
	return linkLineLength;
    }

    public java.awt.Color getLinkLineColor()
    {
	return linkLineColor;
    }

    public int getMessageTrafficLineWidth()
    {
	return messageTrafficLineWidth;
    }

    public java.awt.Color getMessageTrafficToProviderLineColor()
    {
	return messageTrafficToProviderLineColor;
    }

    public java.awt.Color getMessageTrafficToHubLineColor()
    {
	return messageTrafficToHubLineColor;
    }

    public int getArrowheadWidth()
    {
	return arrowheadWidth;
    }

    public int getArrowheadHeight()
    {
	return arrowheadHeight;
    }

    public int getArrowFlashMsec()
    {
	return arrowFlashMsec;
    }

    public int getLabelFontSize()
    {
	return labelFontSize;
    }

    public java.awt.Color getLabelForegroundColor()
    {
	return labelForegroundColor;
    }

    public java.awt.Color getLabelBackgroundColor()
    {
	return labelBackgroundColor;
    }

    public boolean getUseHubOutputDialog()
    {
	return useHubOutputDialog;
    }

    public boolean getUseHubStatusDialog()
    {
	return useHubStatusDialog;
    }

    public java.lang.String getHubPath()
    {
	return hubPath;
    }

    public java.lang.String getProgramFile()
    {
	return programFile;
    }

    public java.lang.String getServerLocationFile()
    {
	return serverLocationFile;
    }

    public java.lang.String getLocations()
    {
	return locations;
    }

    public java.lang.String getInit()
    {
	return init;
    }

    public boolean getDebug()
    {
	return debug;
    }

    public boolean getValidate()
    {
	return validate;
    }

    public int getHubVerbosity()
    {
	return hubVerbosity;
    }

    public java.lang.String getDemoHomePath()
    {
	return demoHomePath;
    }

    public java.lang.String getDemoHomePrefix()
    {
	return demoHomePrefix;
    }

    public void setDrawingAreaWidth(int drawingAreaWidth)
    {
	this.drawingAreaWidth = drawingAreaWidth;
    }

    public void setDrawingAreaHeight(int drawingAreaHeight)
    {
	this.drawingAreaHeight = drawingAreaHeight;
    }

    public void setProviderRadius(int providerRadius)
    {
	this.providerRadius = providerRadius;
    }

    public void setProviderColor(java.awt.Color providerColor)
    {
	this.providerColor = providerColor;
    }

    public void setProviderHighlightColor(java.awt.Color setProviderHighlightColor)
    {
	this.providerHighlightColor = providerHighlightColor;
    }

    public void setProviderFlashMsec(int providerFlashMsec)
    {
	this.providerFlashMsec = providerFlashMsec;
    }

    public void setUnavailableColor(java.awt.Color unavailableColor)
    {
	this.unavailableColor = unavailableColor;
    }

    public void setHubRadius(int hubRadius)
    {
	this.hubRadius = hubRadius;
    }

    public void setHubRunningColor(java.awt.Color hubRunningColor)
    {
	this.hubRunningColor = hubRunningColor;
    }

    public void setHubStoppedColor(java.awt.Color hubStoppedColor)
    {
	this.hubStoppedColor = hubStoppedColor;
    }

    public void setLinkLineWidth(int linkLineWidth)
    {
	this.linkLineWidth = linkLineWidth;
    }

    public void setLinkLineLength(int linkLineLength)
    {
	this.linkLineLength = linkLineLength;
    }

    public void setLinkLineColor(java.awt.Color linkLineColor)
    {
	this.linkLineColor = linkLineColor;
    }

    public void setMessageTrafficLineWidth(int messageTrafficLineWidth)
    {
	this.messageTrafficLineWidth = messageTrafficLineWidth;
    }

    public void setMessageTrafficToProviderLineColor(java.awt.Color messageTrafficToProviderLineColor)
    {
	this.messageTrafficToProviderLineColor = messageTrafficToProviderLineColor;
    }

    public void setMessageTrafficToHubLineColor(java.awt.Color messageTrafficToHubLineColor)
    {
	this.messageTrafficToHubLineColor = messageTrafficToHubLineColor;
    }

    public void setArrowheadWidth(int arrowheadWidth)
    {
	this.arrowheadWidth = arrowheadWidth;
    }

    public void setArrowheadHeight(int arrowheadHeight)
    {
	this.arrowheadHeight = arrowheadHeight;
    }

    public void setArrowFlashMsec(int arrowFlashMsec)
    {
	this.arrowFlashMsec = arrowFlashMsec;
    }

    public void setLabelFontSize(int labelFontSize)
    {
	this.labelFontSize = labelFontSize;
    }

    public void setLabelForegroundColor(java.awt.Color labelForegroundColor)
    {
	this.labelForegroundColor = labelForegroundColor;
    }

    public void setLabelBackgroundColor(java.awt.Color setLabelBackgroundColor)
    {
	this.labelBackgroundColor = labelBackgroundColor;
    }

    public void setUseHubOutputDialog(boolean useHubOutputDialog)
    {
	this.useHubOutputDialog = useHubOutputDialog;
    }

    public void setUseHubStatusDialog(boolean useHubStatusDialog)
    {
	this.useHubStatusDialog = useHubStatusDialog;
    }

    public void setHubPath(java.lang.String hubPath)
    {
	this.hubPath = hubPath;
    }

    public void setProgramFile(java.lang.String programFile)
    {
	this.programFile = programFile;
    }

    public void setServerLocationFile(java.lang.String serverLocationFile)
    {
	this.serverLocationFile = serverLocationFile;
    }

    public void setLocations(java.lang.String locations)
    {
	this.locations = locations;
    }

    public void setInit(java.lang.String init)
    {
	this.init = init;
    }

    public void setDebug(boolean debug)
    {
	this.debug = debug;
    }

    public void setValidate(boolean validate)
    {
	this.validate = validate;
    }

    public void setHubVerbosity(int hubVerbosity)
    {
	this.hubVerbosity = hubVerbosity;
    }

    public void setDemoHomePath(java.lang.String demoHomePath)
    {
	this.demoHomePath = demoHomePath;
    }

    public void setDemoHomePrefix(java.lang.String demoHomePrefix)
    {
	this.demoHomePrefix = demoHomePrefix;
    }
}
